using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.UI.Core;
using System.Windows.Input;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using Fissum.TrayApp.Lib;
using System.Text.RegularExpressions;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Framework.Common;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Fissum.Core.Configuration;
using GalaSoft.MvvmLight.Messaging;
using Microsoft.Practices.Unity;
using Fissum.Core.Repository;

namespace Fissum.TrayApp.Model
{
    internal class ProjectModel
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(ProjectModel));

        public const string FavoriteStatusChangedNotification = "FavoriteStatusChangedNotification";
        public const string ProjectModelMessageToken = "ProjectModelMessageToken";

        private ProjectConfiguration _configuration;
        private Project _project;
        private TPCModel _tpcModel;
        private QueryFolderModel _rootQueryFolder;
        private List<WorkItemTypeModel> _workItemTypeModelList;


        private bool IsLinkedToTFS
        {
            get
            {
                return _project != null && _tpcModel != null && _project.Store.TeamProjectCollection.HasAuthenticated;
            }
        }

        private bool IsLinkedToConfiguration
        {
            get
            {
                return _configuration != null;
            }
        }

        internal Project Project
        {
            get
            {
                return _project;
            }
            set
            {
                if (_project == value)
                    return;
                _project = value;
            }
        }

        private void FillQueryHierarchy()
        {
            if (IsLinkedToTFS)
            {
                _rootQueryFolder = new QueryFolderModel(this, null, null);
                FillQueryHierarchyCore(_rootQueryFolder, _project.QueryHierarchy);
            }
        }

        private void FillQueryHierarchyCore(QueryFolderModel vmFolder, QueryFolder tfsFolder)
        {
            foreach (var item in tfsFolder)
            {
                var tfsSubFolder = item as QueryFolder;
                if (tfsSubFolder != null)
                {
                    var subFolder = new QueryFolderModel(this, tfsSubFolder, vmFolder);
                    vmFolder.SubFolderList.Add(subFolder);
                    FillQueryHierarchyCore(subFolder, item as QueryFolder);
                    continue;
                }

                var tfsQueryDefinition = item as QueryDefinition;
                if (tfsQueryDefinition != null)
                {

                    try
                    {
                        var queryDefinition = new QueryModel(tfsQueryDefinition, this, vmFolder.FullName);
                        vmFolder.QueryList.Add(queryDefinition);
                    }
                    catch
                    {
                        _log.ErrorFormat("Cannot create query '{0}'", tfsQueryDefinition.Name);
                    }

                    
                }
            }
        }


        private void LinkToConfiguration()
        {
            _configuration = ConfigurationRepository.Current.Configuration.CreateProjectConfiguration(_tpcModel.Uri, _project.Name);
        }


        public bool TryLinkToTFS()
        {
            if (!IsLinkedToConfiguration)
            {
                return false;
            }
            var tpcModel = TPCModel.Get(_configuration.TFSUri);

            var otherPM = tpcModel.ProjectList.Where(pm => pm.TFSName == _configuration.Name).FirstOrDefault();
            if (otherPM != null)
            {
                _tpcModel = tpcModel;
                _project = otherPM._project;
                FillQueryHierarchy();
                FillWorkItemTypes();
                return true;
            }
            return false;
        }

        private void FillWorkItemTypes()
        {
            if (IsLinkedToTFS)
            {
                _workItemTypeModelList = (from wit in _project.WorkItemTypes.Cast<WorkItemType>()
                                          orderby wit.Name
                                          select new WorkItemTypeModel(this, wit)).ToList();
            }
            else
            {
                _workItemTypeModelList = new List<WorkItemTypeModel>();
            }
        }

        /// <summary>
        /// Initializes a new instance of the ProjectConfigEditionViewModel class.
        /// </summary>
        /// <param name="project"></param>
        private ProjectModel(Project project, ProjectConfiguration configuration, TPCModel tpcModel)
        {

            if ((tpcModel == null || project == null) && configuration == null)
            {
                throw new ArgumentException("Invalid project construction");
            }
            _tpcModel = tpcModel;
            _project = project;
            _configuration = configuration;
            FillQueryHierarchy();
            FillWorkItemTypes();
        }

        public bool IsConnected
        {
            get
            {
                return _tpcModel != null && _tpcModel.HasAuthenticated;
            }
        }

        public string Name
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.Name;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public string TFSName
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.TFSName;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public bool IsFavorite
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.IsFavorite;
                }
                return false;
            }
            set
            {
                if (!IsLinkedToConfiguration)
                {
                    LinkToConfiguration();
                }

                _configuration.IsFavorite = value;
                var messenger = UnityRepository.Container.Resolve<Messenger>();
                messenger.Send(new NotificationMessage(FavoriteStatusChangedNotification), ProjectModelMessageToken);

            }
        }


        public TPCModel TPCModel
        {
            get
            {
                return _tpcModel;
            }
        }

        public Uri TFSUri
        {
            get
            {
                if (IsLinkedToTFS)
                {
                    return _tpcModel.Uri;
                }
                else
                {
                    return _configuration.TFSUri;
                }
            }
        }

        public static List<ProjectModel> GetFavoriteProjects()
        {
            var favProjectConfigList = ConfigurationRepository.Current.Configuration.GetFavoriteProjects();
            var result = favProjectConfigList.Select(config =>
                {
                    var ret = new ProjectModel(null, config, null);
                    ret.TryLinkToTFS();
                    return ret;
                });
            return result.ToList();
        }

        public void FillQueryHierarchyIfNecessary()
        {
            if (_rootQueryFolder == null && IsLinkedToTFS && IsConnected)
            {
                FillQueryHierarchy();
            }
        }

        public QueryFolderModel QueryRootFolder
        {
            get
            {
                return _rootQueryFolder;
            }
        }


        public List<WorkItemTypeModel> WorkItemTypeModelList
        {
            get
            {
                return _workItemTypeModelList;
            }
        }

        private static Dictionary<string, ProjectModel> _projectModelDictionary;


        static ProjectModel()
        {
            _projectModelDictionary = new Dictionary<string, ProjectModel>();
        }


        public static ProjectModel GetProjetModel(Project project, ProjectConfiguration configuration, TPCModel tpcModel)
        {
            string id = null;
            if (configuration != null)
            {
                id = configuration.TFSUri + "/" + project.Name;
            }
            else
            {
                id = tpcModel.Uri + "/" + project.Name;
            }

            ProjectModel result = null;

            lock (_projectModelDictionary)
            {
                var ret = _projectModelDictionary.TryGetValue(id, out result);
                if (!ret)
                {
                    result = new ProjectModel(project, configuration, tpcModel);
                    _projectModelDictionary.Add(id, result);
                }
            }
            return result;
        }
    }

    internal class WorkItemTypeModel
    {
        private WorkItemType _type;
        private ProjectModel _projectModel;

        /// <summary>
        /// Initializes a new instance of the WorkItemTypeModel class.
        /// </summary>
        /// <param name="projectModel"></param>
        /// <param name="type"></param>
        public WorkItemTypeModel(ProjectModel projectModel, WorkItemType type)
        {

            _type = type;
            _projectModel = projectModel;
        }


        public string Name
        {
            get
            {
                return _type.Name;
            }
        }

        public string Description
        {
            get
            {
                return _type.Description;
            }
        }

        internal WorkItemType Type
        {
            get
            {
                return _type;
            }
        }

        public ProjectModel ProjectModel
        {
            get
            {
                return _projectModel;
            }
        }
    }
}
