using System;
using Fissum.UI.Core;
using Fissum.TrayApp.Model;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using GalaSoft.MvvmLight.Command;
using System.Windows.Input;
using Fissum.TrayApp.Messages;
using GalaSoft.MvvmLight.Messaging;
using Fissum.Core.Repository;
using Microsoft.Practices.Unity;

namespace Fissum.TrayApp.ViewModel
{
    internal class WorkItemViewModel : CoreViewModelBase, IShellViewModel
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(WorkItemViewModel));


        private WorkItemModel _workItemModel;

        public WorkItemModel WorkItemModel
        {
            get
            {
                return _workItemModel;
            }
        }


        public ICommand CloseCommand { get; private set; }
        public ICommand RefreshCommand { get; private set; }
        public ICommand ResetCommand { get; private set; }
        public ICommand SaveCommand { get; private set; }
        public ICommand OpenWorkItemInTSWACommand { get; private set; }

        /// <summary>
        /// Initializes a new instance of the WorkItemViewModel class.
        /// </summary>
        /// <param name="workItemModel"></param>
        public WorkItemViewModel(WorkItemModel workItemModel)
        {
            if (workItemModel == null)
                throw new ArgumentNullException("workItemModel", "workItemModel is null.");
            _workItemModel = workItemModel;
            _workItemModel.FieldChanged += new WorkItemFieldChangeEventHandler(_workItemModel_FieldChanged);
            RefreshCommand = new RelayCommand(() => DoRefresh(), () => !IsModified);
            ResetCommand = new RelayCommand(() => DoReset(), () => IsModified);
            SaveCommand = new RelayCommand(() => DoSave(), () => CanBeSaved);
            OpenWorkItemInTSWACommand = new RelayCommand(() => DoOpenWorkItemInTSWA());
            CloseCommand = new RelayCommand(() => DoClose());
        }

        private bool? AskUser(string title, string message)
        {
            var msg = new UserSimpleRequestMessage()
            {
                Title = title,
                Message = message
            };

            var messenger = UnityRepository.Container.Resolve<Messenger>();

            messenger.Send(msg);

            return msg.Response;
        }


        private void DoRefresh()
        {
            WorkItemModel.Refresh();
        }


        private void DoClose()
        {
            var handler = RequestClose;
            if (handler != null)
            {
                handler(this, EventArgs.Empty);
            }
        }

        private void DoReset()
        {
            if (AskUser("Confirmation", "Would you like to revert your changes?") == true)
            {
                WorkItemModel.Reset();
            }
        }

        private void DoOpenWorkItemInTSWA()
        {
            var uri = _workItemModel.GetTSWAUri();
            if (uri != null)
            {
                var messenger = UnityRepository.Container.Resolve<Messenger>();
                messenger.Send(new OpenLinkInBrowserMessage(uri));
            }
        }
        private void DoSave()
        {
            WorkItemModel.Save(new WorkItemModel[] { WorkItemModel });
        }

        void _workItemModel_FieldChanged(object sender, WorkItemEventArgs e)
        {
            //if (e.Field.ReferenceName == "System.Title")
            //{
            //    RaisePropertyChangedSafe(() => ShellTitle);
            //}
            RaisePropertyChangedSafe(() => IsModified);
            RaisePropertyChangedSafe(() => CanBeSaved);
            RaisePropertyChangedSafe(() => IsValid);
            RaisePropertyChangedSafe(() => ShellTitle);
            RaisePropertyChangedSafe(() => HasSaveError);
        }

        public bool IsModified
        {
            get
            {
                return _workItemModel.IsModified;
            }
        }

        public bool CanBeSaved
        {
            get
            {
                return _workItemModel.CanBeSaved;
            }
        }
        public bool HasSaveError
        {
            get
            {
                return _workItemModel.HasSaveError;
            }

        }
        public bool IsValid
        {
            get
            {
                return _workItemModel.IsValid;
            }

        }

        public string WorkItemType
        {
            get
            {
                return _workItemModel.WorkItem.Type.Name;
            }
        }

        public override void Cleanup()
        {
            base.Cleanup();
            _workItemModel.FieldChanged -= new WorkItemFieldChangeEventHandler(_workItemModel_FieldChanged);
        }

        #region IShellViewModel Members

        public string ShellTitle
        {
            get
            {
                if (_workItemModel.IsNew)
                {
                    return string.Format("{0} :: New {1}", _workItemModel.WorkItem.Project.Name, _workItemModel.WorkItem.Type.Name);
                }
                else
                {
                    var result = string.Format("{0} :: #{1} {2}", _workItemModel.WorkItem.Project.Name, _workItemModel.Id.Id, _workItemModel.WorkItem.Title);
                    if (IsModified)
                    {
                        result += " (modified)";
                    }
                    return result;
                }
            }
        }

        public object ShellIcon
        {
            get { return Program.Application.TFS16Image; }
        }

        public CloseResult QueryClose()
        {
            if (IsModified)
            {
                if (AskUser("Confirmation", "Save result before closing?") == true)
                {
                    WorkItemModel.Save(new WorkItemModel[] { _workItemModel });
                    return _workItemModel.IsModified ? CloseResult.Cancel : CloseResult.Close;
                }
                else
                {
                    _workItemModel.Reset();
                    return CloseResult.Close;
                }
            }
            else
            {
                return CloseResult.Close;
            }
        }
        #endregion

        #region IShellViewModel Members


        public event EventHandler RequestClose;

        #endregion
    }
}
