using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Diagnostics;

namespace Fissum.Models
{
    [DebuggerDisplay("{Path}")]
    public class NodeModel
    {
        internal const string AreaRootNodeName = "Project Structure";
        internal const string IteratorRootNodeName = "Project Iteration";
        private ProjectModel _project;
        private Node _node;

        private List<NodeModel> _childNodeList;

        public NodeModel(ProjectModel project, Node node)
        {
            _project = project;
            _node = node;
            BuildNodeCollection();
        }

        public NodeModel(ProjectModel project, Node node, List<NodeModel> nodes)
        {
            _project = project;
            _node = node;
            _childNodeList = nodes;
        }

        private void BuildNodeCollection()
        {
            if (_childNodeList != null)
            {
                return;
            }
            _childNodeList = new List<NodeModel>();

            if (!_node.HasChildNodes)
            {
                return;
            }
            _childNodeList = BuildNodeList(_project, _node.ChildNodes);
        }


        internal static List<NodeModel> BuildNodeList(ProjectModel project, NodeCollection collection)
        {
            List<NodeModel> result = new List<NodeModel>();

            foreach (Node node in collection)
            {
                result.Add(new NodeModel(project, node));
            }

            return result;
        }

        /// <summary>
        /// Recursive method used to find a node from its id
        /// </summary>
        /// <param name="nodeId"></param>
        /// <returns></returns>
        public NodeModel Find(int nodeId)
        {
            if (nodeId == _node.Id)
            {
                return this;
            }

            return NodeModel.Find(_childNodeList, nodeId);
        }

        public static NodeModel Find(IEnumerable<NodeModel> nodes, int nodeId)
        {
            foreach (NodeModel childNode in nodes)
            {
                NodeModel result;

                result = childNode.Find(nodeId);

                if (result != null)
                {
                    return result;
                }
            }
            return null;
        }

        public string Path
        {
            get
            {
                if (_node != null)
                {
                    return _node.Path;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public string Name
        {
            get
            {
                if (_node != null)
                {
                    if ((_node.Name == AreaRootNodeName) ||
                        (_node.Name == IteratorRootNodeName))
                    {
                        return _project.Name;
                    }
                    else
                    {
                        return _node.Name;
                    }
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public NodeModel[] ChildNodes
        {
            get
            {
                return _childNodeList.ToArray();

            }
        }

        public int Id
        {
            get
            {
                if (_node != null)
                {
                    return _node.Id;
                }
                else
                {
                    return _project.Project.Id;
                }
            }
        }

        /// <summary>
        /// Indicates if the given node is contained in the current node hierarchy
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool Contains(NodeModel node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            return Contains(node.Id);
        }

        /// <summary>
        /// Indicates whether the current node is under the specified node or not
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool IsUnder(NodeModel node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }

            return node.Contains(this);

        }

        /// <summary>
        /// indicates if the given node id is contained in the current node hierarchy
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool Contains(int nodeId)
        {
            return Find(nodeId) != null;
        }

        #region IWorkItemCoreValueTranslator Members

        public object GetWorkItemCoreValue()
        {
            return _node.Path;
        }

        #endregion
    }
}
