using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using GalaSoft.MvvmLight.Messaging;
using System.Windows.Media;
using Fissum.Core.Configuration;
using Fissum.Core.Repository;
using Microsoft.Practices.Unity;
using Microsoft.TeamFoundation.Server;
using System.Diagnostics;
namespace Fissum.Models
{
    [DebuggerDisplay("{Name}")]
    public class ProjectModel
    {

        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(ProjectModel));

        public const string FavoriteStatusChangedNotification = "FavoriteStatusChangedNotification";
        public const string ProjectRefreshing = "ProjectRefreshingNotification";
        public const string ProjectRefreshed = "ProjectRefreshedNotification";
        public const string ProjectModelMessageToken = "ProjectModelMessageToken";

        private ProjectConfiguration _configuration;
        private Project _project;
        private TPCModel _tpcModel;
        private QueryFolderModel _rootQueryFolder;
        private List<WorkItemTypeModel> _workItemTypeModelList;
        private List<NodeModel> _areaNodeList;
        private List<NodeModel> _iterationNodeList;

        private bool IsLinkedToTFS
        {
            get
            {
                return _project != null && _tpcModel != null && _project.Store.TeamProjectCollection.HasAuthenticated;
            }
        }

        private bool IsLinkedToConfiguration
        {
            get
            {
                return _configuration != null;
            }
        }

        public Color? ColorTag
        {
            get
            {
                if (!IsLinkedToConfiguration)
                {
                    return null;
                }
                return _configuration.Color;
            }
            set
            {
                TryLinkToConfiguration();
                _configuration.Color = value;
            }
        }

        internal Project Project
        {
            get
            {
                return _project;
            }
            set
            {
                if (_project == value)
                    return;
                _project = value;
            }
        }

        private void FillQueryHierarchy()
        {
            if (IsLinkedToTFS)
            {
                _rootQueryFolder = new QueryFolderModel(this, null, null);
                try
                {
                    FillQueryHierarchyCore(_rootQueryFolder, _project.QueryHierarchy);
                }
                catch
                {

                }
            }
        }


        private void FillQueryHierarchyCore(QueryFolderModel vmFolder, QueryFolder tfsFolder)
        {
            foreach (var item in tfsFolder)
            {
                var tfsSubFolder = item as QueryFolder;
                if (tfsSubFolder != null)
                {
                    var subFolder = new QueryFolderModel(this, tfsSubFolder, vmFolder);
                    vmFolder.SubFolderList.Add(subFolder);
                    FillQueryHierarchyCore(subFolder, item as QueryFolder);
                    continue;
                }

                var tfsQueryDefinition = item as QueryDefinition;
                if (tfsQueryDefinition != null)
                {

                    try
                    {
                        var queryDefinition = new QueryModel(tfsQueryDefinition, this, vmFolder.FullName);
                        vmFolder.QueryList.Add(queryDefinition);
                    }                 
                    catch
                    {
                        _log.ErrorFormat("Cannot create query '{0}'", tfsQueryDefinition.Name);
                    }

                }
            }
        }


        private void TryLinkToConfiguration()
        {
            if (_configuration == null)
            {
                _configuration = ConfigurationRepository.Current.Configuration.CreateProjectConfiguration(_tpcModel.Uri, _project.Name);
            }
        }


        public bool TryLinkToTFS()
        {
            if (!IsLinkedToConfiguration)
            {
                return false;
            }
            var tpcModel = TPCModel.Get(_configuration.TFSUri);
            if (tpcModel.State != TPCState.UpToDate)
            {
                return false;
            }
            var otherPM = tpcModel.ProjectList.Where(pm => pm.TFSName == _configuration.Name).FirstOrDefault();
            if (otherPM != null)
            {
                _tpcModel = tpcModel;
                _project = otherPM._project;
                FillQueryHierarchy();
                FillWorkItemTypes();
                FillProperties();
                return true;
            }
            return false;
        }

        private void FillWorkItemTypes()
        {
            if (IsLinkedToTFS)
            {
                _workItemTypeModelList = (from wit in _project.WorkItemTypes.Cast<WorkItemType>()
                                          orderby wit.Name
                                          select new WorkItemTypeModel(this, wit)).ToList();
            }
            else
            {
                _workItemTypeModelList = new List<WorkItemTypeModel>();
            }
        }

        private void FillProperties()
        {
            string projectName;
            string state;
            int templateId;
            ProjectProperty[] properties;
            _tpcModel.ICSS.GetProjectProperties(_project.Uri.ToString(), out projectName, out state, out templateId, out properties);
        }
        /// <summary>
        /// Initializes a new instance of the ProjectConfigEditionViewModel class.
        /// </summary>
        /// <param name="project"></param>
        private ProjectModel(Project project, ProjectConfiguration configuration, TPCModel tpcModel)
        {

            if ((tpcModel == null || project == null) && configuration == null)
            {
                throw new ArgumentException("Invalid project construction");
            }
            _tpcModel = tpcModel;
            _project = project;
            _configuration = configuration;
            FillQueryHierarchy();
            FillWorkItemTypes();
            FillIterationAndAreaHierarchy();
        }

        public bool IsConnected
        {
            get
            {
                return _tpcModel != null && _tpcModel.HasAuthenticated;
            }
        }

        public string Name
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.Name;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public string TFSName
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.TFSName;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public Project TFSProject
        {
            get            
            {
                return _project;
            }

        }

        public bool IsFavorite
        {
            get
            {
                if (IsLinkedToConfiguration)
                {
                    return _configuration.IsFavorite;
                }
                return false;
            }
            set
            {
                if (!IsLinkedToConfiguration)
                {
                    TryLinkToConfiguration();
                }

                _configuration.IsFavorite = value;
                var messenger = UnityRepository.Container.Resolve<Messenger>();
                messenger.Send(new NotificationMessage(FavoriteStatusChangedNotification), ProjectModelMessageToken);

            }
        }


        public TPCModel TPCModel
        {
            get
            {
                return _tpcModel;
            }
        }

        public Uri TFSUri
        {
            get
            {
                if (IsLinkedToTFS)
                {
                    return _tpcModel.Uri;
                }
                else
                {
                    return _configuration.TFSUri;
                }
            }
        }

        public Uri ProjectUri
        {
            get
            {
                if (IsLinkedToTFS)
                {
                    return new Uri(TFSUri, _project.Name);
                }
                else
                {
                    return new Uri(TFSUri, _configuration.TFSName);
                }
            }
        }

        public static List<ProjectModel> GetFavoriteProjects()
        {
            var favProjectConfigList = ConfigurationRepository.Current.Configuration.GetFavoriteProjects();
            var result = favProjectConfigList.Select(config =>
            {
                var ret = new ProjectModel(null, config, null);
                ret.TryLinkToTFS();
                return ret;
            });
            return result.ToList();
        }

        public void FillQueryHierarchyIfNecessary()
        {
            if (_rootQueryFolder == null && IsLinkedToTFS && IsConnected)
            {
                FillQueryHierarchy();
            }
        }

        public QueryFolderModel QueryRootFolder
        {
            get
            {
                return _rootQueryFolder;
            }
        }


        public List<WorkItemTypeModel> WorkItemTypeModelList
        {
            get
            {
                return _workItemTypeModelList;
            }
        }

        private static Dictionary<string, ProjectModel> _projectModelDictionary;



        static ProjectModel()
        {
            _projectModelDictionary = new Dictionary<string, ProjectModel>();
        }


        public static ProjectModel GetProjetModel(Project project, ProjectConfiguration configuration, TPCModel tpcModel)
        {
            string id = null;
            if (configuration != null)
            {
                id = configuration.TFSUri + "/" + project.Name;
            }
            else
            {
                id = tpcModel.Uri + "/" + project.Name;
            }

            ProjectModel result = null;

            lock (_projectModelDictionary)
            {
                var ret = _projectModelDictionary.TryGetValue(id, out result);
                if (!ret)
                {
                    result = new ProjectModel(project, configuration, tpcModel);
                    _projectModelDictionary.Add(id, result);
                }
            }
            return result;
        }

        internal void Refresh(bool refreshCache)
        {
            if (refreshCache)
            {
                _project.Store.RefreshCache();
            }
            FillWorkItemTypes();
            FillQueryHierarchy();
            FillIterationAndAreaHierarchy();
        }

        private void FillIterationAndAreaHierarchy()
        {
            _areaNodeList = new List<NodeModel>();


            if (_project != null && _project.AreaRootNodes.Count > 0)
            {
                Node node = _project.AreaRootNodes[0];
                node = node.ParentNode;
                if ((node.Name == NodeModel.AreaRootNodeName) && (node.ParentNode != null))
                {
                    node = node.ParentNode;
                    _areaNodeList.Add(new NodeModel(this, node, NodeModel.BuildNodeList(this, _project.AreaRootNodes)));
                }

            }
            _iterationNodeList = new List<NodeModel>();

            if (_project != null && _project.IterationRootNodes.Count > 0)
            {
                Node node = _project.IterationRootNodes[0];
                node = node.ParentNode;
                if ((node.Name == NodeModel.IteratorRootNodeName) && (node.ParentNode != null))
                {
                    node = node.ParentNode;
                    _iterationNodeList.Add(new NodeModel(this, node, NodeModel.BuildNodeList(this, _project.IterationRootNodes)));
                }
            }
        }
    }
}
