using System;
using System.Collections.Generic;
using System.Linq;
using Fissum.Core;
using Fissum.Core.Configuration;
using Fissum.Core.Repository;
using GalaSoft.MvvmLight.Messaging;
using log4net;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.Practices.Unity;
using Fissum.Models.Messages;
using Microsoft.TeamFoundation.Server;
using System.Diagnostics;

namespace Fissum.Models
{
    [DebuggerDisplay("{Name}")]
    public class TPCModel : IEquatable<TPCModel>
    {
        private static ILog _log = LogManager.GetLogger(typeof(TPCModel));

        private TfsTeamProjectCollection _tpc;
        private WorkItemStore _workItemStore;
        private VersionControlServer _versionControlServer;
        private ICommonStructureService _icss;
        private TswaClientHyperlinkService _tswaService;
        private TPCConfiguration _configuration;


        public TPCTFSObjects TFS { get; private set; }
        public List<ProjectModel> ProjectList { get; private set; }
        public TPCState State { get; private set; }
        /// <summary>
        /// Initializes a new instance of the TPCModel class.
        /// </summary>
        /// <param name="tpc"></param>
        public TPCModel(TfsTeamProjectCollection tpc, TPCConfiguration config)
        {
            if (tpc == null)
                throw new ArgumentNullException("tpc", "tpc is null.");
            if (config == null)
                throw new ArgumentNullException("config", "config is null.");
            _tpc = tpc;

            _configuration = config;
            ProjectList = new List<ProjectModel>();
            TFS = new TPCTFSObjects(this);
        }

        private void FillProjectList()
        {           
            var projects = _versionControlServer.GetAllTeamProjects(true);
            _workItemStore.RefreshCache();
            var teamExplorerProjects = TeamExplorerConfiguration.Instance.GetProjectUris(_tpc);
            foreach (TeamProject pinfo in projects)
            {
                Project project = _workItemStore.Projects[pinfo.Name];

                ProjectModel model = ProjectList
                .Where(pm => pm.Project.Id == project.Id)
                .FirstOrDefault();
                if (model == null)
                {
                    if (teamExplorerProjects == null || teamExplorerProjects.Contains(project.Uri))
                    {
                        var projectConfig = _configuration.GetProjectConfiguration(project.Name);
                        ProjectList.Add(ProjectModel.GetProjetModel(project, projectConfig, this));
                    }
                }
                else
                {
                    model.Refresh(refreshCache: false);
                }
            }
        }

        private void InitializeServices()
        {
            _versionControlServer = _tpc.GetService<VersionControlServer>();
            _icss = _tpc.GetService<ICommonStructureService>();
            if (_tpc.Name.ToLowerInvariant().Contains("codeplex"))
            {
                _tswaService = null;
            }
            else
            {
                _tswaService = _tpc.GetService<TswaClientHyperlinkService>();
                try
                {
                    var uri = _tswaService.GetHomeUrl();
                    if (uri == null)
                    {
                        _tswaService = null;
                    }
                }
                catch (Exception ex)
                {
                    _log.Warn(string.Format("TSWA Service for server '{0}' not available", _tpc.Uri), ex);
                    _tswaService = null;
                }
            }
        }

        public bool UpdateTPC()
        {
            try
            {
                if (_tpc.HasAuthenticated)
                {
                    InitializeServices();
                    SetState(TPCState.Updating);
                    FillProjectList();
                }
                SetState(TPCState.UpToDate);
                return true;
            }
            catch (Exception ex)
            {
                _log.Warn("Cannont update TPC", ex);
                SetState(TPCState.Disconnected);
                return false;
            }
        }
        public string Identity
        {
            get
            {
                return _tpc.AuthorizedIdentity.DisplayName;
            }
        }

        public bool Connect()
        {

            if (_tpc.HasAuthenticated)
            {
                return true;
            }
            try
            {
                SetState(TPCState.Connecting);
                _log.DebugFormat("{0}: Connecting", Name);
                _tpc.Authenticate();
                _workItemStore = new WorkItemStore(_tpc);
                
            }
            catch (Exception ex)
            {
                _log.Error(string.Format("{0}: authentication failed: {1}", Name, ex.Message), ex);
                SetState(TPCState.Disconnected);
            }
            var messenger = UnityRepository.Container.Resolve<Messenger>();

            messenger.Send(new TPCAuthenticationMessage(this, _tpc.HasAuthenticated), this.Uri);
            if (!_tpc.HasAuthenticated)
            {
                return false;
            }
            
            _log.DebugFormat("{0}: Connected", Name);
            return UpdateTPC();
        }


        private void SetState(TPCState newState)
        {
            if (State != newState)
            {
                State = newState;

                var messenger = UnityRepository.Container.Resolve<Messenger>();
                messenger.Send(new TPCStateMessage(this, newState), this.Uri);
            }
        }

        public bool AutoConnect
        {
            get
            {
                return _configuration.AutoConnect;
            }
            set
            {
                _configuration.AutoConnect = value;
            }
        }


        public bool HasAuthenticated
        {
            get
            {
                return _tpc.HasAuthenticated;
            }
        }


        public string Name
        {
            get
            {
                return _tpc.Name;
            }
        }

        public Uri Uri
        {
            get
            {
                return _tpc.Uri;
            }
        }

        public bool IsTSWAConfigured
        {
            get
            {
                return _tswaService != null;
            }
        }

        #region IEquatable<TPCModel> Members

        public bool Equals(TPCModel other)
        {
            if (other == null)
            {
                return false;
            }

            return _tpc.Uri.Equals(other._tpc.Uri);
        }

        #endregion

        #region TSWA

        public Uri GetTSWAUrl(QueryModel queryModel)
        {
            if (_tswaService == null)
            {
                return null;
            }
            var result = _tswaService.GetWorkItemQueryResultsUrl(queryModel.ProjectModel.Project.Uri, queryModel.Definition.Id);

            return result;
        }
        public Uri GetTSWAUrl(WorkItemModel workItemModel)
        {
            if (_tswaService == null)
            {
                return null;
            }

            var result = _tswaService.GetWorkItemEditorUrl(workItemModel.Id.Id);

            return result;
        }

        #endregion


        #region statics

        private static Dictionary<Uri, TPCModel> _tpcModelDictionary;

        static TPCModel()
        {
            _tpcModelDictionary = new Dictionary<Uri, TPCModel>();
        }

        internal TfsTeamProjectCollection TPC
        {
            get
            {
                return _tpc;
            }
        }

        internal static TPCModel Get(Uri uri)
        {
            if (uri == null)
                throw new ArgumentNullException("uri", "uri is null.");

            TPCModel result = null;

            if (!_tpcModelDictionary.TryGetValue(uri, out result))
            {
                var tpc = TPCRepository.Current[uri];

                if (tpc == null)
                {
                    result = null;
                }
                else
                {
                    throw new InvalidOperationException("Unknown TPC");
                }
            }
            return result;
        }

        public static IEnumerable<TPCModel> GetTPCModels()
        {
            return _tpcModelDictionary.Values;
        }

        public static IEnumerable<TPCModel> InitializeFromConfig()
        {
            var q = from tpcConfig in ConfigurationRepository.Current.ActualTPCConfigList
                    let tpc = TPCRepository.Current[tpcConfig.Uri]
                    where tpc != null
                    select new TPCModel(tpc, tpcConfig);

            _tpcModelDictionary = q.ToDictionary(model => model.Uri);

            return q.ToList();
        }

        #endregion



        internal TfsTeamProjectCollection TeamProjectCollection
        {
            get
            {
                return _tpc;
            }
        }

        internal WorkItemStore WorkItemStore
        {
            get
            {
                return _workItemStore;
            }
        }


        internal VersionControlServer VersionControlServer
        {
            get
            {
                return _versionControlServer;
            }

        }

        internal ICommonStructureService ICSS
        {
            get
            {
                return _icss;
            }
        }

    }
}