using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Xml.XPath;
using Fissum.Core;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.Models
{
    public class WorkItemModel
    {
        #region Statics

        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(WorkItemModel));

        private static Dictionary<WorkItemModelId, WeakReference<WorkItemModel>> _modelDictionary;

        static WorkItemModel()
        {
            _modelDictionary = new Dictionary<WorkItemModelId, WeakReference<WorkItemModel>>();
        }

        public static WorkItemModel Create(WorkItemTypeModel typeModel)
        {
            var newWorkItem = typeModel.Type.NewWorkItem();

            var result = new WorkItemModel(newWorkItem, typeModel.ProjectModel.TPCModel);
            return result;
        }

        public static WorkItemModel Get(WorkItemModelId wmid)
        {
            return Get(wmid.Id, wmid.TPCModel);
        }

        public static List<WorkItemModel> GetFromWorkItemCollection(WorkItemCollection collection, TPCModel tpcModel)
        {
            return GetFromWorkItems(collection.Cast<WorkItem>().ToList(), tpcModel);
        }
        public static List<WorkItemModel> GetFromWorkItems(IList<WorkItem> collection, TPCModel tpcModel)
        {
            if (tpcModel == null)
                throw new ArgumentNullException("tpcModel", "tpcModel is null.");
            if (collection == null)
                throw new ArgumentNullException("collection", "collection is null.");
            return collection
            .Select(wi => GetFromWorkItem(wi, tpcModel))
            .ToList();
        }

        public static WorkItemModel GetFromWorkItem(WorkItem workItem, TPCModel tpcModel)
        {
            if (tpcModel == null)
                throw new ArgumentNullException("tpcModel", "tpcModel is null.");
            if (workItem == null)
                throw new ArgumentNullException("workItem", "workItem is null.");

            var wId = new WorkItemModelId(workItem.Id, tpcModel);

            WeakReference<WorkItemModel> result = null;

            _modelDictionary.TryGetValue(wId, out result);

            if ((result == null) || (!result.IsAlive))
            {

                var wmodel = new WorkItemModel(workItem, tpcModel);
                result = new WeakReference<WorkItemModel>(wmodel);
                _modelDictionary[wId] = result;
            }
            else
            {
                if (!result.Target.IsModified && workItem.Revision > result.Target.WorkItem.Revision)
                {
                    result.Target.Refresh();
                }
            }

            return result.Target;
        }

        public static WorkItemModel Get(int id, TPCModel tpcModel)
        {
            if (tpcModel == null)
                throw new ArgumentNullException("tpcModel", "tpcModel is null.");
            if (!tpcModel.HasAuthenticated)
                return null;

            var wId = new WorkItemModelId(id, tpcModel);

            WeakReference<WorkItemModel> result = null;

            _modelDictionary.TryGetValue(wId, out result);

            if ((result == null) || (!result.IsAlive))
            {
                var store = new WorkItemStore(tpcModel.TPC);
                var wi = store.GetWorkItem(id);
                var wmodel = new WorkItemModel(wi, tpcModel);
                result = new WeakReference<WorkItemModel>(wmodel);
                _modelDictionary[wId] = result;
            }
            else
            {
                if (!result.Target.IsModified)
                {
                    result.Target.WorkItem.SyncToLatest();
                }
            }

            return result.Target;
        }
        public static void Save(IEnumerable<WorkItemModel> workitems)
        {
            var wimNewToSave = from wi in workitems
                               where wi.IsNew
                               select wi;

            foreach (var wim in wimNewToSave)
            {
                wim.WorkItem.Save(SaveFlags.MergeLinks);
                var weakRef = new WeakReference<WorkItemModel>(wim);
                _modelDictionary[wim.Id] = weakRef;
                wim.OnFieldChanged(wim.WorkItem, new WorkItemEventArgs(wim.WorkItem.Fields["System.Rev"]));
            }

            var wiGroupsToSave = from wi in workitems
                                 where !wi.IsNew
                                 group wi by wi.TPCModel
                                 into gp
                                 select new
                                 { TCPModel = gp.Key, WorkItems = gp.ToArray() };

            foreach (var group in wiGroupsToSave)
            {
                var store = group.WorkItems[0].WorkItem.Store;
                var wis = group.WorkItems.Select(wim => wim.WorkItem).ToArray();
                var errors = store.BatchSave(wis, SaveFlags.MergeLinks);

                foreach (var wim in group.WorkItems)
                {
                    if (wim.IsNew)
                        continue;

                    var error = errors.FirstOrDefault(e => e.WorkItem.Id == wim.Id.Id);
                    if (error == null)
                    {
                        wim.OnFieldChanged(wim.WorkItem, new WorkItemEventArgs(wim.WorkItem.Fields["System.Rev"]));
                    }
                    else
                    {
                        wim.HasSaveError = true;
                        wim.OnFieldChanged(wim.WorkItem, new WorkItemEventArgs(wim.WorkItem.Fields["System.Id"]));
                    }
                }
            }
        }
        #endregion


        private WorkItem _workItem;
        private TPCModel _tpcModel;
        private ProjectModel _projectModel;

        private bool _containsWorkflowControl;

        /// <summary>
        /// Initializes a new instance of the WorkItemModel class.
        /// </summary>
        /// <param name="workItem"></param>
        /// <param name="tpcModel"></param>
        private WorkItemModel(WorkItem workItem, TPCModel tpcModel)
        {
            _workItem = workItem;
            _tpcModel = tpcModel;
            _workItem.FieldChanged += new WorkItemFieldChangeEventHandler(OnFieldChanged);
            GrabInfoFromWorkItem();
        }

        public event WorkItemFieldChangeEventHandler FieldChanged;

        void OnFieldChanged(object sender, WorkItemEventArgs e)
        {
            if (e.Field == null)
            {
                return;
            }
            var handle = FieldChanged;
            if (handle != null)
            {
                FieldChanged(this, e);
            }
        }

        private void GrabInfoFromWorkItem()
        {
            try
            {
                _containsWorkflowControl = false;
                var doc = XDocument.Parse(_workItem.DisplayForm, LoadOptions.None);
                var result = doc.Root.XPathSelectElement("//Control[@Type='Workflow']");
                _containsWorkflowControl = result != null;

            }
            catch (Exception ex)
            {
                _log.Error("Grabbing info from workitem process template failed", ex);
            }
        }

        public Uri GetTSWAUri()
        {
            return _tpcModel.GetTSWAUrl(this);
        }


        public WorkItem WorkItem
        {
            get
            {
                return _workItem;
            }
        }


        public ProjectModel ProjectModel
        {
            get
            {
                if (_projectModel == null)
                {

                    _projectModel = _tpcModel.ProjectList
                    .Where(pm => pm.Name == _workItem.Project.Name)
                    .First();
                }
                return _projectModel;
            }
        }

        public bool ContainsWorkflowControl
        {
            get
            {
                return _containsWorkflowControl;
            }
        }

        public TPCModel TPCModel
        {
            get
            {
                return _tpcModel;
            }
        }


        WorkItemModelId _id;
        public WorkItemModelId Id
        {
            get
            {
                if (IsNew)
                {
                    return null;
                }

                if (_id == null)
                {
                    _id = new WorkItemModelId(_workItem.Id, _tpcModel);
                }
                return _id;
            }

        }

        public bool IsNew
        {
            get
            {
                return _workItem.IsNew;
            }
        }

        public bool IsModified
        {
            get
            {
                return _workItem.IsDirty;
            }
        }


        public bool IsValid
        {
            get
            {
                return _workItem.IsValid();
            }
        }

        public string GetErrors()
        {
            return _workItem.Validate().ToString();
        }

        public void Reset()
        {
            _workItem.Reset();
            HasSaveError = false;
            OnFieldChanged(WorkItem, new WorkItemEventArgs(WorkItem.Fields["System.Id"]));
        }

        public bool IsReadOnly
        {
            get
            {
                return _workItem.IsReadOnly;
            }
        }


        public bool CanBeSaved
        {
            get
            {
                return !HasSaveError && !_workItem.IsReadOnly && _workItem.IsDirty && _workItem.IsValid();
            }
        }

        public bool HasSaveError
        {
            get; private set;
        }

        public string GetValidationMessage()
        {
            List<string> errorMsgList = new List<string>();
            var arrayList = _workItem.Validate();
            if (arrayList.Count == 0)
            {
                return  String.Empty;
            }

            foreach (var field in arrayList.Cast<Field>())
            {
                string errorMsg;

                switch (field.Status)
                {
                    case FieldStatus.InvalidCharacters:
                        errorMsg = "Invalid characters";
                        break;
                    case FieldStatus.InvalidComputedField:
                        errorMsg = "Invalid computed field";
                        break;
                    case FieldStatus.InvalidDate:
                        errorMsg = "Invalid date";
                        break;
                    case FieldStatus.InvalidEmpty:
                        errorMsg = "Field must not be empty";
                        break;
                    case FieldStatus.InvalidEmptyOrOldValue:
                        errorMsg = "Field empty or old value";
                        break;
                    case FieldStatus.InvalidFormat:
                        errorMsg = "Invalid format";
                        break;
                    case FieldStatus.InvalidPath:
                        errorMsg = "Invalid path";
                        break;
                    case FieldStatus.InvalidTooLong:
                        errorMsg = "Value too long";
                        break;
                    case FieldStatus.InvalidType:
                        errorMsg = "Invalid type";
                        break;

                    case FieldStatus.InvalidUnknown:
                    case FieldStatus.InvalidValueInOtherField:
                    case FieldStatus.InvalidValueNotInOtherField:
                    case FieldStatus.Valid:
                    case FieldStatus.InvalidListValue:
                    case FieldStatus.InvalidNotEmpty:
                    case FieldStatus.InvalidNotEmptyOrOldValue:
                    case FieldStatus.InvalidNotOldValue:
                    case FieldStatus.InvalidOldValue:
                    default:
                        errorMsg = "Invalid value";
                        break;
                }

                errorMsgList.Add(String.Format("Field '{0}': {1}", field.Name, errorMsg));
            }

            return String.Join("; ", errorMsgList.ToArray());
        }


        public void Refresh()
        {
            _workItem.SyncToLatest();
            OnFieldChanged(WorkItem, new WorkItemEventArgs(WorkItem.Fields["System.Id"]));

        }
        public WorkItemModel[] GetChilds()
        {
            var q = from WorkItemLink link in _workItem.WorkItemLinks
                    select WorkItemModel.Get(link.TargetId, _tpcModel);

            return q.ToArray();
        }
    }



}
