﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.UI.Core;
using System.Windows.Input;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using Fissum.TrayApp.Lib;
using System.Text.RegularExpressions;
using Microsoft.TeamFoundation.Client;
using Fissum.Core.Configuration;
using Fissum.UI.Core.Menu;
using GalaSoft.MvvmLight.Command;
using Fissum.Models;
using GalaSoft.MvvmLight.Messaging;
using Fissum.Core.Repository;
using Microsoft.Practices.Unity;
using Fissum.TrayApp.Messages;
using Fissum.UI.Core.ViewModel;

namespace Fissum.TrayApp.ViewModel
{
	internal class MainViewModel : CoreViewModelBase, IShellViewModel
	{
		private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(MainViewModel));
		
		private List<IMenuItem> _menuItems;
		private List<IMenuItem> _favoriteMenuItems;
		private List<IMenuItem> _staticCommandsMenuItems;
		private ICommand _aboutFissumCommand;
		private SimpleMenuItem _serversMenuItem;

		private void BuildFavoriteMenuItems()
		{
			_favoriteMenuItems = new List<IMenuItem>();

			var favoriteProjects = (from p in ProjectModel.GetFavoriteProjects()
									orderby p.Name
									select new ProjectMenuItem(p,false)).ToList();

			if (favoriteProjects.Count > 0)
			{
				_favoriteMenuItems.AddRange(favoriteProjects);
				_favoriteMenuItems.Add(new SeparatorMenuItem());
			}

		}

		private void BuildStaticCommands()
		{
			_staticCommandsMenuItems = new List<IMenuItem>();
			var models = TPCModel.GetTPCModels();
			var q = from tcpModel in models
					select new TPCMenuItem(tcpModel, this);

			_serversMenuItem = new SimpleMenuItem()
			{
				Header = "Project Collections",
				Icon = Program.Application.TPC16Image
			};
			_serversMenuItem.Items = q.ToList();
			_staticCommandsMenuItems.Add(_serversMenuItem);
			_staticCommandsMenuItems.Add(new SeparatorMenuItem());

			var runtimeConfig = UnityRepository.Container.Resolve<RuntimeConfiguration>();
			if (runtimeConfig.IsInDebugMode)
			{
				_staticCommandsMenuItems.Add(new SimpleMenuItem()
				{
					Header = "Debug window",
					Icon = Program.Application.Bug16Image,
					Command = new RelayCommand(DoShowDebugWindow)
				});
			}

			_staticCommandsMenuItems.Add(new SimpleMenuItem()
			{
				Header = "About Fissum...",
				Icon = Program.Application.Info16Image,
				Command = new RelayCommand(DoShowAboutFissumWindow)
			});
			_staticCommandsMenuItems.Add(new SimpleMenuItem()
			{
				Header = "Quit",
				Command = new RelayCommand(DoQuitFissum)
			});

		}

		private void RebuildMenuItems()
		{
			_menuItems = new List<IMenuItem>();
			_menuItems.AddRange(_favoriteMenuItems);
			_menuItems.AddRange(_staticCommandsMenuItems);
			RaisePropertyChangedSafe(() => MenuItems);
		}

		private void InitializeViewModel()
		{
			_aboutFissumCommand = new RelayCommand(DoShowAboutFissumWindow);
			BuildFavoriteMenuItems();
			BuildStaticCommands();
			RebuildMenuItems();
		  

		
			foreach (var model in TPCModel.GetTPCModels())
			{                
				if (model.AutoConnect)
				{
					var modelToLoad = model;
					this.ProcessAsync(() => modelToLoad.Connect());
				}
			}
		}





		private void ProcessNotification(NotificationMessage notification)
		{
			if (notification.Notification == ProjectModel.FavoriteStatusChangedNotification)
			{
				ProcessOnUIThread(() =>
					{
						BuildFavoriteMenuItems();
						RebuildMenuItems();
					});
			}
		}


		private void InitializeEvents()
		{
 var messenger = UnityRepository.Container.Resolve<Messenger>();

 messenger.Register<NotificationMessage>(this, ProjectModel.ProjectModelMessageToken, notification => ProcessNotification(notification));
		}

		public MainViewModel()
			: base()
		{
			InitializeViewModel();
			InitializeEvents();
		}

		private void DoShowDebugWindow()
		{
			var messenger = UnityRepository.Container.Resolve<Messenger>();
			messenger.Send(new ShowDebugWindowMessage());
		}

		private void DoShowAboutFissumWindow()
		{
			var messenger = UnityRepository.Container.Resolve<Messenger>();
			messenger.Send(new ShowAboutWindowMessage());            
		}

		private void DoQuitFissum()
		{
			Program.Application.Shutdown();
		}



		public ICommand LoadedCommand { get; private set; }

		public IEnumerable<IMenuItem> MenuItems
		{
			get
			{
				return _menuItems;
			}
		}

		#region IShellViewModel Members

		public string ShellTitle
		{
			get { return "Fissum"; }
		}

		public object ShellIcon
		{
			get { return null; }
		}

		public CloseResult QueryClose()
		{
			return CloseResult.Close;  
		}

		public event EventHandler RequestClose;

		#endregion
	}
}
