using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.UI.Core;
using System.Windows.Input;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using Fissum.TrayApp.Lib;
using System.Text.RegularExpressions;
using Microsoft.TeamFoundation.Client;
using Fissum.Core.Configuration;
using Fissum.UI.Core.Menu;
using GalaSoft.MvvmLight.Command;
using Fissum.Models;
using Fissum.TrayApp.Messages;
using Microsoft.Practices.Unity;
using Fissum.Core.Repository;
using GalaSoft.MvvmLight.Messaging;
using System.Windows.Media;
using Fissum.Models.Messages;
using Fissum.UI.Core.Messages;

namespace Fissum.TrayApp.ViewModel
{
    internal class ProjectMenuItem : MenuItemBase
    {
        private static readonly log4net.ILog _log = log4net.LogManager.GetLogger(typeof(ProjectMenuItem));



        private SimpleMenuItem _setAsFavoriteMenuItem;
        private SimpleMenuItem _removeAsFavoriteMenuItem;
        private SimpleMenuItem _setColorTagMenuItem;

        private bool _showFavoriteIcon;

        public ProjectModel ProjectModel { get; private set; }

        /// <summary>
        /// Initializes a new instance of the ProjectMenuItem class.
        /// </summary>
        public ProjectMenuItem(ProjectModel projectModel, bool showFavoriteIcon)
        {
            ProjectModel = projectModel;
            IsUpdating = false;
            _showFavoriteIcon = showFavoriteIcon;
            IsConnected = projectModel.IsConnected;
            var messenger = UnityRepository.Container.Resolve<Messenger>();
            if (IsConnected)
            {
                Icon = Program.Application.TFS16Image;
            }
            else
            {
                Icon = Program.Application.TFS16DisabledImage;
            }

            messenger.RegisterEx<TPCStateMessage>(this, projectModel.TFSUri,null, HandleStateMessage,EventExecutionThreadOption.OnUIThread);
            messenger.RegisterEx<NotificationMessage>(this, projectModel.ProjectUri, null,HandleNotificationMessage,EventExecutionThreadOption.OnUIThread);

            _setColorTagMenuItem = new SimpleMenuItem();
            _setColorTagMenuItem.Header = "Set Color Tag...";
            _setColorTagMenuItem.Icon = Program.Application.ColorWheel16Image;
            _setColorTagMenuItem.Command = new RelayCommand(() => DoSetColorTag());

            FillSubmenu();
        }


        public string Name
        {
            get
            {
                return ProjectModel.Name;
            }
        }

        public bool IsConnected
        {
            get;
            private set;
        }

        public bool IsUpdating
        {
            get;
            private set;
        }

        public bool ShowFavorite
        {
            get
            {
                return ProjectModel.IsFavorite && _showFavoriteIcon;
            }
        }

        public Brush ColorTagBrush
        {
            get
            {
                if (ProjectModel.ColorTag.HasValue)
                {
                    return new SolidColorBrush(ProjectModel.ColorTag.Value);
                }
                {
                    return null;
                }
            }
        }

        public bool HasColorTag
        {
            get
            {
                return ProjectModel.ColorTag.HasValue;
            }
        }


        public bool IsFavorite
        {
            get
            {
                return ProjectModel.IsFavorite;
            }
        }


        private List<IMenuItem> ProcessQueryFolder(QueryFolderModel folder)
        {
            if (folder == null)
                throw new ArgumentNullException("folder", "folder is null.");


            List<IMenuItem> result = new List<IMenuItem>();

            foreach (QueryFolderModel subFolder in folder.SubFolderList)
            {
                var item = new SimpleMenuItem();
                item.Header = subFolder.Name;
                item.Icon = Program.Application.Folder16Image;
                item.Items = ProcessQueryFolder(subFolder);
                result.Add(item);
            }

            foreach (QueryModel query in folder.QueryList)
            {
                var item = new QueryMenuItem(query);
                result.Add(item);
            }

            return result;

        }

        private void FillSubmenu()
        {
            var newItemList = new List<IMenuItem>();
            ProjectModel.FillQueryHierarchyIfNecessary();
            if (IsConnected)
            {
                if (ProjectModel.QueryRootFolder != null)
                {
                    var queries = ProcessQueryFolder(ProjectModel.QueryRootFolder);
                    newItemList.AddRange(queries);
                    newItemList.Add(new SeparatorMenuItem());
                }

                var workItemCreateMenuItem = new SimpleMenuItem();
                workItemCreateMenuItem.Header = "New Work Items";
                workItemCreateMenuItem.Icon = Program.Application.New16Image;
                workItemCreateMenuItem.Items = (from witm in ProjectModel.WorkItemTypeModelList
                                                select new WorkItemTypeMenuItem(witm)).ToList();

                newItemList.Add(workItemCreateMenuItem);

                newItemList.Add(new SeparatorMenuItem());
            }




            if (_setAsFavoriteMenuItem == null)
            {
                _setAsFavoriteMenuItem = new SimpleMenuItem();
                _setAsFavoriteMenuItem.Header = "Set as favorite";
                _setAsFavoriteMenuItem.Visible = !IsFavorite;
                _setAsFavoriteMenuItem.Icon = Program.Application.Favorite16Image;
                _setAsFavoriteMenuItem.Command = new RelayCommand(() =>
                    {
                        DoSetAsFavorite();
                    });
            }
            newItemList.Add(_setAsFavoriteMenuItem);

            if (_removeAsFavoriteMenuItem == null)
            {
                _removeAsFavoriteMenuItem = new SimpleMenuItem();
                _removeAsFavoriteMenuItem.Header = "Remove from favorite";
                _removeAsFavoriteMenuItem.Visible = IsFavorite;
                _removeAsFavoriteMenuItem.Icon = Program.Application.NotFavorite16Image;
                _removeAsFavoriteMenuItem.Command = new RelayCommand(() =>
                {
                    RemoveAsFavorite();
                });
            }
            newItemList.Add(_removeAsFavoriteMenuItem);

            newItemList.Add(_setColorTagMenuItem);


            var addinCommands = Program.Application.AddinManager.GetMenuItemsForProject(this.ProjectModel);
            if (addinCommands.Count > 0)
            {
                newItemList.Add(new SeparatorMenuItem());
                newItemList.AddRange(addinCommands);

            }

            Items = newItemList;

            RaisePropertyChangedSafe(() => Items);
        }

        private void DoSetAsFavorite()
        {
            ProjectModel.IsFavorite = true;
            RaisePropertyChangedSafe(() => ShowFavorite);
        }


        private void RemoveAsFavorite()
        {
            ProjectModel.IsFavorite = false;
            RaisePropertyChangedSafe(() => ShowFavorite);
        }

        private void DoSetColorTag()
        {
            var chooseColorViewModel = new ChooseColorViewModel();
            try
            {
                if (ProjectModel.ColorTag.HasValue)
                {
                    chooseColorViewModel.R = ProjectModel.ColorTag.Value.R;
                    chooseColorViewModel.G = ProjectModel.ColorTag.Value.G;
                    chooseColorViewModel.B = ProjectModel.ColorTag.Value.B;
                }
                var messenger = UnityRepository.Container.Resolve<Messenger>();

                var msg = new ShowDialogMessage(chooseColorViewModel);
                messenger.Send(msg);

                if (msg.DialogResult == true)
                {
                    var newColor = Color.FromRgb(chooseColorViewModel.R, chooseColorViewModel.G, chooseColorViewModel.B);
                    ProjectModel.ColorTag = newColor;

                    RaisePropertyChangedSafe(() => ColorTagBrush);
                    RaisePropertyChangedSafe(() => HasColorTag);
                }
            }
            finally
            {
                chooseColorViewModel.Cleanup();
            }

        }

        private void HandleNotificationMessage(NotificationMessage message)
        {
            switch (message.Notification)
            {
            	case ProjectModel.ProjectRefreshing:
            		  IsUpdating = true;
            		break;
                case ProjectModel.ProjectRefreshed:
                     IsUpdating = false;
                    break;
                default:
                    break;
            }

            RaisePropertyChangedSafe(() => IsUpdating);
            FillSubmenu();
        }

        private void HandleStateMessage(TPCStateMessage message)
        {
            var wasConnected = IsConnected;
            switch (message.State)
            {
                case TPCState.Disconnected:
                    IsConnected = false;
                    IsUpdating = false;
                    break;
                case TPCState.Connecting:
                    IsConnected = false;
                    IsUpdating = true;
                    break;
                case TPCState.Updating:
                    IsConnected = true;
                    IsUpdating = true;
                    break;
                case TPCState.UpToDate:
                    IsConnected = true;
                    IsUpdating = false;
                    break;
                case TPCState.Dirty:
                default:
                    IsConnected = false;
                    IsUpdating = false;
                    break;

            }



            RaisePropertyChangedSafe(() => IsConnected);
            RaisePropertyChangedSafe(() => IsUpdating);
            if (IsConnected && !IsUpdating)
            {
                if (ProjectModel.TryLinkToTFS())
                {
                    RaisePropertyChangedSafe(() => Name);
                }
            }
            if (IsConnected)
            {
                Icon = Program.Application.TFS16Image;
            }
            else
            {
                Icon = Program.Application.TFS16DisabledImage;
            }
            FillSubmenu();
          
        }





    }
}
