using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.TeamSystem;
using System.Collections;
using System.Linq.Expressions;
using System.Reflection;
using Fissum.Core.Linq;

namespace Fissum.TeamSystem
{
    /// <summary>
    /// Query extender used to provider LINQ capability to the TFS model
    /// </summary>
    public static class QueryExtender
    {
        /// <summary>
        /// Extends the server for a LINQ Query on the work item store.
        /// </summary>
        /// <param name="server">The server.</param>
        /// <returns></returns>
        public static Query<WorkItemObject> WorkItems(this TeamFoundationServer server)
        {
            if (server == null)
            {
                throw new ArgumentNullException("server");
            }

            return new Query<WorkItemObject>(new WorkItemLinqQueryProvider<WorkItemObject>(server));
        }

        /// <summary>
        /// Extends the project for a LINQ Query on the work item store.
        /// </summary>
        /// <param name="project">The project.</param>
        /// <returns></returns>
        public static Query<WorkItemObject> WorkItems(this Project project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            project.Load();
            return new Query<WorkItemObject>(new WorkItemLinqQueryProvider<WorkItemObject>(project,null));
        }

        /// <summary>
        /// Extends the project for a specific work item type LINQ Query.
        /// </summary>
        /// <typeparam name="T">Type of work item</typeparam>
        /// <typeparam name="U">Related creator provider</typeparam>
        /// <param name="project">The project.</param>
        /// <returns></returns>
        public static Query<T> Get<T, U>(this Project project) where T: WorkItemObject where U: IWorkItemCreatorProvider<T>, new()
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            project.Load();

            return new Query<T>(new WorkItemLinqQueryProvider<T>(project,new U()));
        }

        /// <summary>
        /// Extends the project for a specific work item type LINQ Query.
        /// </summary>
        /// <typeparam name="T">Type of work item</typeparam>
        /// <param name="project">The project.</param>
        /// <returns></returns>
        public static Query<T> Get<T>(this Project project) where T : WorkItemObject
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            project.Load();

            if (!WorkItemPropertyUtility<T>.IsValid)
            {
                throw new InvalidOperationException(String.Format("Type '{0}' does not have a the needed attributes", typeof(T)));
            }

            return new Query<T>(new WorkItemLinqQueryProvider<T>(project, WorkItemPropertyUtility<T>.Provider));
        }

        /// <summary>
        /// Check if a work item is of a the specified type
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="wi"></param>
        /// <returns></returns>
        public static bool Is<T>(this WorkItemObject wi) where T : WorkItemObject
        {          
            if (!WorkItemPropertyUtility<T>.IsValid)
            {
                throw new InvalidOperationException(String.Format("Type '{0}' does not have a the needed attributes", typeof(T)));
            }

            return wi.WIType.Name == WorkItemPropertyUtility<T>.WorkItemTypeName;
        }

        /// <summary>
        /// Cast a work item. 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="wi">The wi.</param>
        /// <returns></returns>
        public static T As<T>(this WorkItemObject wi) where T: WorkItemObject
        {
            if (!Is<T>(wi))
            {
                return null;
            }

            if (!WorkItemPropertyUtility<T>.IsValid)
            {
                throw new InvalidOperationException(String.Format("Type '{0}' does not have a the needed attributes", typeof(T)));
            }

            return WorkItemPropertyUtility<T>.Provider.CreateItem(wi.TFSWorkItem, wi.Project.Server);
        }

        public static bool Check<T>(this Project project) where T : WorkItemObject
        {
            return WorkItemPropertyUtility<T>.CheckProjectUsability(project);
        }
    }
}
