using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Fissum.TeamSystem;
using System.Collections;
using System.Linq.Expressions;
using System.Reflection;
using Fissum.Core.Linq;
using Fissum.TeamSystem.Model;

namespace Fissum.TeamSystem
{
    /// <summary>
    /// Utility class used to discover and manipulate specialized workitem 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public static class WorkItemPropertyUtility<T> where T : WorkItemObject
    {
        internal class PropertyData
        {
            public string ReferenceName { get; set; }
            public Type Type { get; set; }
        }

        private static IWorkItemCreatorProvider<T> _provider;
        private static string _workItemTypeName;
        private static bool _typeProcessed = false;
        private static List<PropertyData> _propertyDataList;
        private static bool _propertyProcessed = false;


        /// <summary>
        /// Processes the work item type and extracts the useful data from its properties
        /// </summary>
        static void ProcessProperties()
        {
            if (_propertyProcessed)
            {
                return;
            }
            _propertyProcessed = true;
            _propertyDataList = new List<WorkItemPropertyUtility<T>.PropertyData>();


            PropertyInfo[] props = typeof(T).GetProperties(BindingFlags.Instance | BindingFlags.Public | BindingFlags.DeclaredOnly);

            foreach (PropertyInfo prop in props)
            {
                FieldAttribute[] fieldAttribs = prop.GetCustomAttributes(typeof(FieldAttribute), false) as FieldAttribute[];

                if (fieldAttribs == null || fieldAttribs.Length != 1)
                {
                    continue;
                }

                _propertyDataList.Add(new PropertyData() { ReferenceName = fieldAttribs[0].ReferenceName, Type = prop.PropertyType });
            }
        }

        /// <summary>
        /// Processes the work item type and extracts the useful data
        /// </summary>
        static void ProcessType()
        {
            if (_typeProcessed)
            {
                return;
            }
            _typeProcessed = true;
            ModelTypeAttribute[] modelAttribs = typeof(T).GetCustomAttributes(typeof(ModelTypeAttribute), false) as ModelTypeAttribute[];

            if (modelAttribs == null || modelAttribs.Length != 1)
            {
                _workItemTypeName = null;
            }
            else
            {
                _workItemTypeName = modelAttribs[0].WIType;
            }

            CreatorProviderAttribute[] providerAttribs = typeof(T).GetCustomAttributes(typeof(CreatorProviderAttribute), false) as CreatorProviderAttribute[];

            if (providerAttribs == null || providerAttribs.Length != 1)
            {
                _provider = null;
            }
            else
            {
                try
                {
                    _provider = Activator.CreateInstance(providerAttribs[0].CreatorType) as IWorkItemCreatorProvider<T>;
                }
                catch
                {
                    _provider = null;
                }
            }
        }

        /// <summary>
        /// Gets the provider.
        /// </summary>
        /// <value>The provider.</value>
        public static IWorkItemCreatorProvider<T> Provider
        {
            get
            {
                ProcessType();
                return _provider;
            }
        }

        /// <summary>
        /// Gets the name of the work item type.
        /// </summary>
        /// <value>The name of the work item type.</value>
        public static string WorkItemTypeName
        {
            get
            {
                ProcessType();
                return _workItemTypeName;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the work item is valid for Linq to WIQL queries
        /// </summary>
        /// <value><c>true</c> if this instance is valid; otherwise, <c>false</c>.</value>
        public static bool IsValid
        {
            get
            {
                ProcessType();
                return _provider != null && !String.IsNullOrEmpty(_workItemTypeName);
            }
        }

        public static bool MatchWorkItemType(WorkItemType type)
        {
            if (type == null)
            {
                return false;
            }            

            if (type.Name != WorkItemTypeName)
            {
                return false;
            }

            ProcessProperties();

            foreach (var propData in _propertyDataList)
            {
                var matchingField = (from field in type.Fields
                                     where field.ReferenceName == propData.ReferenceName
                                     select field).FirstOrDefault();

                if (matchingField == null)
                {
                    return false;
                }

                if (propData.Type == matchingField.FieldType)
                {
                    continue;
                }

                if (matchingField.FieldType.IsClass)
                {
                    return false;
                }

                Type nullableType = typeof(Nullable<>).MakeGenericType(new Type [] { matchingField.FieldType });
                if (propData.Type == nullableType)
                {
                    continue;
                }
                return false;
            }
            return true;
        }

        /// <summary>
        /// Verifies if the project can be used for the current work item type
        /// </summary>
        /// <param name="project">The project.</param>
        /// <returns></returns>
        public static bool CheckProjectUsability(Project project)
        {
            foreach (WorkItemType wiType in project.Types)
            {
                if (wiType.Name == WorkItemTypeName)
                {
                    return MatchWorkItemType(wiType);
                }
            }
            return false;
        }
    }
}
