﻿using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Markup;
using Fissum.Model.Generator;

namespace Fissum.Model.Builder
{
    internal class Program
    {
        private const string MODEL_FILE_PARAM_NAME = "model";
        private const string CODE_PARAN_NAME = "code";
        private const string OUTPUT_PARAM_NAME = "output";
        private const string DESIGNER_NAMEMODE = "designer";

        private const string CS_CODE_MODEL = "cs";
        private const string CPP_CODE_MODEL = "cpp";
        private const string VB_CODE_MODEL = "vb";
        private const string HelpText = @"Command line arguments:
To show this help: -h
To build a C# file: -model modelfile.wimd -code cs -output file.cs
To build a VB.Net file: -model modelfile.wimd -code vb -output file.vb
To build a file from the model file: mymodelName.wind -> mymodelname.(cs|vb), just remove the '-output ...' part
To build a mymodelname.Designer.(cs|vb): add '-designer' to the command line
";

        /// <summary>
        /// Main method. 
        /// </summary>
        /// <param name="args">The args.</param>
        private static void Main(string[] args)
        {
            string codeModel = CS_CODE_MODEL;

            Arguments commandLine = new Arguments(args);

            if (commandLine["h"] != null)
            {
                Console.WriteLine(HelpText);
                return;
            }

            commandLine.TryGetParam(CODE_PARAN_NAME, ref codeModel);
            string codeExtension = null;
            CodeDomProvider provider;
            switch (codeModel)
            {
                case CS_CODE_MODEL:
                    provider = new Microsoft.CSharp.CSharpCodeProvider();
                    codeExtension = ".cs";
                    break;
                case VB_CODE_MODEL:
                    provider = new Microsoft.VisualBasic.VBCodeProvider();
                    codeExtension = ".vb";
                    break;
                default:
                    provider = null;
                    break;
            }


            if (provider == null)
            {
                ShowErrorAndQuit(String.Format("Invalid provider: '{0}'", codeModel));
                return;
            }

            string modelName = commandLine[MODEL_FILE_PARAM_NAME];
            string outputName = commandLine[OUTPUT_PARAM_NAME];

            if (String.IsNullOrEmpty(modelName))
            {
                ShowErrorAndQuit("Model file is missing");
                return;
            }

            if (String.IsNullOrEmpty(outputName))
            {
                if (commandLine[DESIGNER_NAMEMODE] != null)
                {
                    outputName = Path.Combine(Path.GetDirectoryName(modelName), String.Format("{0}.Designer{1}", Path.GetFileNameWithoutExtension(modelName), codeExtension));
                }
                else
                {
                    outputName = Path.ChangeExtension(modelName, codeExtension);
                }               
            }
            else
            {
                string outputExtension = Path.GetExtension(outputName);
                if (String.IsNullOrEmpty(outputExtension))
                {
                    Path.ChangeExtension(outputName, outputExtension);
                }
                else if (outputExtension != codeExtension)
                {
                    ShowErrorAndQuit(String.Format("Invalid output extension '{0}'. It should be '{1}'", outputExtension, codeExtension));
                }
            }

            ModelDefinition modelDefinition = null;
            try
            {
                Stream stream = File.OpenRead(modelName);
                using (stream)
                {
                    modelDefinition = ModelDefinition.LoadFromStream(stream);
                }


            }
            catch (FileLoadException)
            {
                ShowErrorAndQuit("Cannont open the model file");
                return;
            }
            catch
            {
                ShowErrorAndQuit("Cannont load the model file");
                return;
            }


            if (modelDefinition == null)
            {
                ShowErrorAndQuit("The model definition is invalid");
            }

            Engine engine = new Engine();


            StreamWriter writer = new StreamWriter(outputName);

            using (writer)
            {
                engine.GenerateModel(modelDefinition, provider, writer);
                writer.Flush();
                writer.Close();
            }
            Environment.ExitCode = 0;

        }

        /// <summary>
        /// Shows the error and quit.
        /// </summary>
        /// <param name="msg">The MSG.</param>
        /// <param name="exitCode">The exit code.</param>
        private static void ShowErrorAndQuit(string msg, int exitCode)
        {
            ShowError(msg);
            Environment.Exit(exitCode);

        }
        /// <summary>
        /// Shows the error and quit. The exit code is "-1".
        /// </summary>
        /// <param name="msg">The MSG.</param>
        private static void ShowErrorAndQuit(string msg)
        {
            int exitCode = -1;
            ShowErrorAndQuit(msg, exitCode);
        }
        
        /// <summary>
        /// Shows the error with a specific color.
        /// </summary>
        /// <param name="msg">The MSG.</param>
        private static void ShowError(string msg)
        {
            ConsoleColor currentColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine(msg);

            Console.ForegroundColor = currentColor;
        }
    }


}
