﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Fissum.Model.Generator;
using Fissum.Core.Configuration;

namespace Fissum.Model.BuilderUI
{
    /// <summary>
    /// Form used to choice between opening an existing model or create a new one.
    /// </summary>
    internal partial class ConnectingForm : Form
    {
        private Fissum.Core.Context _context;
        private ModelInfo _model;
        ModelDefinition _modelDefinition;

        /// <summary>
        /// Initializes a new instance of the <see cref="ConnectingForm"/> class.
        /// </summary>
        /// <param name="context">The context.</param>
        public ConnectingForm(Fissum.Core.Context context)
            : base()
        {
            _context = context;
            BuilderStatus = BuilderStartupStatus.NewModel;
            InitializeComponent();
        }

        /// <summary>
        /// Handles the Load event of the ConnectingForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ConnectingForm_Load(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(ModelToOpenName))
            {
                try
                {
                    _modelDefinition = ModelDefinition.LoadFromFile(ModelToOpenName);
                }
                catch
                {
                    MessageBox.Show("Error while loading Model file");
                    Application.Exit();
                }
                LoadFormData();
            }
            else
            {
                _context.ConnectAsync(delegate
                {
                    this.Invoke(new MethodInvoker(delegate
                    {

                        LoadFormData();
                    }));
                });
            }                           
        }

        /// <summary>
        /// Prepares the form for the user choice.
        /// </summary>
        public void LoadFormData()
        {
            if (_context.Servers.Count() == 0)
            {
                MessageBox.Show("No server available");
                Application.Exit();
            }
            this.FormBorderStyle = FormBorderStyle.FixedDialog;
            this.ControlBox = true;
            
            Text = "Connected";
            progressBar.Style = ProgressBarStyle.Continuous;
            progressBar.Value = 100;
            if (String.IsNullOrEmpty(ModelToOpenName))
            {
              
                bsServer.DataSource = (from server in _context.Servers                                                
                                                  select server).ToList();
                this.actionPanel.Enabled = true;                
            }
            else
            {
                ModelDefinition modelDefinition = null;
                try
                {
                    var server = _context.GetServer(_modelDefinition.Server);
                    if (!server.Connected)
                    {
                        if (server.Connect())
                        {
                            MessageBox.Show("Can't connect to the server");
                            Application.Exit();
                        }
                    }
                    modelDefinition = ModelDefinition.LoadFromFile(ModelToOpenName);
                    _model = ModelInfo.BuildFromModelDefinition(_context, modelDefinition);
                }
                catch
                {
                    _model = null;
                }
                if (_model == null)
                {
                    MessageBox.Show("Error while loading Model file");
                    Application.Exit();
                }
                
                DialogResult = DialogResult.OK;
            }
        }

        public BuilderStartupStatus BuilderStatus { get; set; }
        public string ModelToOpenName { get; set; }
        internal TeamSystem.TeamFoundationServer SelectedServer
        {
            get
            {
                return cbServer.SelectedItem as TeamSystem.TeamFoundationServer;
            }
        }

        internal TeamSystem.Project SelectedProject
        {
            get
            {
                return cbProject.SelectedItem as TeamSystem.Project;

            }
        }

        public ModelInfo ModelInfo
        {
            get
            {
                return _model;
            }
        }

        private void CbNewModel_Click(object sender, EventArgs e)
        {
            if (SelectedProject == null || SelectedServer == null)
            {
                return;
            }

            _model = ModelInfo.BuildFromProject(SelectedProject);
            DialogResult = DialogResult.OK;
        }

        private void CbLoadModel_Click(object sender, EventArgs e)
        {
            if (ofdModel.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            ModelDefinition modelDefinition = null;

            try
            {
                modelDefinition = ModelDefinition.LoadFromFile(ofdModel.FileName);
            }
            catch
            {
                modelDefinition = null;
            }
            if (modelDefinition == null)
            {
                MessageBox.Show("Error while opening modelfile");
            }

            _model = ModelInfo.BuildFromModelDefinition(_context,modelDefinition);
            DialogResult = DialogResult.OK;
        }

        private void CbProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            cbNewModel.Enabled = SelectedProject != null && SelectedServer != null;
        }
    }
}
