using System;
using System.Collections.Generic;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.TeamSystem
{
    public class Node : IWorkItemCoreValueTranslator
    {
        internal const  string AreaRootNodeName = "Project Structure";
        internal const string IteratorRootNodeName = "Project Iteration";
        private Project _project;
        private TFSTracking.Node _node;

        private List<Node> _childNodeList;

        public Node(Project project, TFSTracking.Node node)
        {
            _project = project;
            _node = node;
            BuildNodeCollection();
        }

        public Node(Project project, TFSTracking.Node node,List<Node> nodes)
        {
            _project = project;
            _node = node;
            _childNodeList = nodes;
        }

        private  void BuildNodeCollection()
        {
            if (_childNodeList != null)
            {
                return;
            }
            _childNodeList = new List<Node>();

            if (!_node.HasChildNodes)
            {
                return;
            }
            _childNodeList = BuildNodeList(_project, _node.ChildNodes);
        }
        

        internal static List<Node> BuildNodeList(Project project, TFSTracking.NodeCollection collection)
        {
            List<Node> result = new List<Node>();

            foreach (TFSTracking.Node node in collection)
            {
                result.Add(new Node(project,node));
            }

            return result;
        }

        /// <summary>
        /// Recursive method used to find a node from its id
        /// </summary>
        /// <param name="nodeId"></param>
        /// <returns></returns>
        public Node Find(int nodeId)
        {
            if (nodeId == _node.Id)
            {
                return this;
            }

            return Node.Find(_childNodeList, nodeId);
        }

        public static Node Find(IEnumerable<Node> nodes, int nodeId)
        {
            foreach (Node childNode in nodes)
            {
                Node result;

                result = childNode.Find(nodeId);

                if (result != null)
                {
                    return result;
                }
            }
            return null;
        }

        public string Path
        {
            get
            {
                if (_node != null)
                {
                    return _node.Path;
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public string Name
        {
            get
            {
                if (_node != null)
                {
                    if ((_node.Name == AreaRootNodeName) ||
                        (_node.Name == IteratorRootNodeName))
                    {
                        return _project.Name;
                    }
                    else
                    {
                        return _node.Name;
                    }
                }
                else
                {
                    return _project.Name;
                }
            }
        }

        public Node[] ChildNodes
        {
            get
            {
                return _childNodeList.ToArray();

            }
        }

        public int Id
        {
            get
            {
                if (_node != null)
                {
                    return _node.Id;
                }
                else
                {
                    return _project.TFSProject.Id;
                }
            }
        }

        /// <summary>
        /// Indicates if the given node is contained in the current node hierarchy
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool Contains(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            return Contains(node.Id);
        }

        /// <summary>
        /// Indicates whether the current node is under the specified node or not
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool IsUnder(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }

            return node.Contains(this);

        }

        /// <summary>
        /// indicates if the given node id is contained in the current node hierarchy
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public bool Contains(int nodeId)
        {
            return Find(nodeId) != null;
        }

        #region IWorkItemCoreValueTranslator Members

        public object GetWorkItemCoreValue()
        {
            return _node.Path;
        }

        #endregion
    }
}