using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Text;
using System.Xml;
using Microsoft.TeamFoundation.Server;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.TeamSystem
{
    public sealed class Project: IEquatable<Project>, IWorkItemCoreValueTranslator
    {
        private TeamFoundationServer _server;
        private TFSTracking.Project _project;
        private string _name;

        private List<WorkItemQuery> _queryList;
        private List<WorkItemType> _typeList;
        private List<Node> _areaNodeList;
        private List<Node> _iterationNodeList;
        private List<WorkItemFieldDefinition> _fieldList;
        private List<DocumentList> _docList;
        private bool _wssSiteOnline;

        private WorkItemFieldDefinition _areaField;
        private WorkItemFieldDefinition _iterationField;
        private bool _loaded;

        /// <summary>
        /// Load the document lists of the current project.
        /// </summary>
        public void LoadProjectDocuments()
        {
            _docList.Clear();
            WSList.Lists wsList= new Fissum.TeamSystem.WSList.Lists();
            using (wsList)
            {
                wsList.Url = ProjectSiteUrl + @"/_vti_bin/lists.asmx";

                wsList.Credentials = _server.TFServer.Credentials;          

                XmlNode nodeListCollection;
                try
                {
                    nodeListCollection = wsList.GetListCollection();
                }
                catch
                {
                    _wssSiteOnline = false;
                    return;
                }
                _wssSiteOnline = true;




                XmlNamespaceManager nsmgr = new XmlNamespaceManager(nodeListCollection.OwnerDocument.NameTable);
                nsmgr.AddNamespace("base", nodeListCollection.NamespaceURI);
                XmlNodeList lists = nodeListCollection.SelectNodes("base:List[@BaseType='1' and @ServerTemplate='101']", nsmgr);

                for (int i = 0; i < lists.Count; i++)
                {
                    string listTitle = lists[i].Attributes["Title"].Value;
                    // Get details
                    XmlNode nodeList = wsList.GetList(listTitle);
                    Guid listId = new Guid(nodeList.Attributes["ID"].Value);
                    string rootFolder = nodeList.Attributes["RootFolder"].Value;


                    DocumentList docList = new DocumentList(this, wsList, listTitle, rootFolder, listId);
                    _docList.Add(docList);
                }            
            }                   
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Project"/> class.
        /// </summary>
        /// <param name="server">The server.</param>
        /// <param name="project">The project.</param>
        internal Project(TeamFoundationServer server, TFSTracking.Project project)
        {
            if (server == null)
            {
                throw new ArgumentNullException("server");
            }
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            _server = server;
            _project = project;
            _name = project.Name;
            _queryList = new List<WorkItemQuery>();
            _docList = new List<DocumentList>();
            _wssSiteOnline = false;
            _loaded = false;                  
        }

        public void Load()
        {
            if (_loaded)
            {
                return;
            }
            
            foreach (TFSTracking.StoredQuery query in _project.StoredQueries)
            {
                _queryList.Add(new WorkItemQuery(query, this));
            }
            _typeList = new List<WorkItemType>();

            foreach (TFSTracking.WorkItemType t in _project.WorkItemTypes)
            {
                _typeList.Add(new WorkItemType(t, this));
            }

            _fieldList = new List<WorkItemFieldDefinition>();

            foreach (TFSTracking.FieldDefinition field in _project.Store.FieldDefinitions)
            {
                WorkItemFieldDefinition wiField = new WorkItemFieldDefinition(field);

                switch (wiField.ReferenceName)
                {
                    case WorkItemFieldDefinition.ReferenceNameArea:

                        _areaField = wiField;
                        break;
                    case WorkItemFieldDefinition.ReferenceNameIteration:
                        _iterationField = wiField;
                        break;
                }

                _fieldList.Add(wiField);
            }

            _areaNodeList = new List<Node>();


            if (_project.AreaRootNodes.Count > 0)
            {
                TFSTracking.Node node = _project.AreaRootNodes[0];
                node = node.ParentNode;
                if ((node.Name == Node.AreaRootNodeName) && (node.ParentNode != null))
                {
                    node = node.ParentNode;
                    _areaNodeList.Add(new Node(this, node, Node.BuildNodeList(this, _project.AreaRootNodes)));
                }

            }
            _iterationNodeList = new List<Node>();
           
            if (_project.IterationRootNodes.Count > 0)
            {
                TFSTracking.Node node = _project.IterationRootNodes[0];
                node = node.ParentNode;
                if ((node.Name == Node.IteratorRootNodeName) && (node.ParentNode != null))
                {
                    node = node.ParentNode;
                    _iterationNodeList.Add(new Node(this, node, Node.BuildNodeList(this, _project.IterationRootNodes)));
                }
            }
            _loaded = true;
        }


        public bool Loaded
        {
            get
            {
                return _loaded;
            }
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return _name;
            }
        }

        /// <summary>
        /// Gets the queries.
        /// </summary>
        /// <value>The queries.</value>
        public WorkItemQuery [] Queries
        {
            get
            {
                Load();
                return _queryList.ToArray();
            }
        }

        /// <summary>
        /// Gets the types.
        /// </summary>
        /// <value>The types.</value>
        public WorkItemType[] Types
        {
            get
            {
                Load();
                return _typeList.ToArray();
            }
        }

        public Node[] AreaNodes
        {
            get
            {
                Load();
                return _areaNodeList.ToArray();
            }
        }

        public Node[] IterationNodes
        {
            get
            {
                Load();
                return _iterationNodeList.ToArray();
            }
        }

        public WorkItemFieldDefinition[] QueryFields
        {
            get
            {
                Load();
                return _fieldList.ToArray();
            }

        }

        /// <summary>
        /// Gets the server.
        /// </summary>
        /// <value>The server.</value>
        public TeamFoundationServer Server
        {
            get
            {                
                return _server;
            }
        }

        /// <summary>
        /// Gets the area field.
        /// </summary>
        /// <value>The area field.</value>
        public WorkItemFieldDefinition AreaField
        {
            get
            {
                Load();
                return _areaField;
            }
        }

        /// <summary>
        /// Gets the iteration field.
        /// </summary>
        /// <value>The iteration field.</value>
        public WorkItemFieldDefinition IterationField
        {
            get
            {
                Load();
                return _iterationField;
            }
        }

        /// <summary>
        /// Gets the TFS project.
        /// </summary>
        /// <value>The TFS project.</value>
        internal TFSTracking.Project TFSProject
        {
            get
            {
                return _project;
            }

        }

        /// <summary>
        /// Gets the project site URL.
        /// </summary>
        /// <value>The project site URL.</value>
        public string ProjectSiteUrl
        {
            get
            {
                Load();
                return _server.WssSiteBaseUrl + "/" + _project.Name;
            }
        }

        public string ProjectSiteUnc
        {
            get
            {
                Load();
                return _server.WssSiteBaseUnc + @"\" + _project.Name;
            }

        }

        /// <summary>
        /// Gets a value indicating whether WSS site is online.
        /// </summary>
        /// <value><c>true</c> if [WSS site online]; otherwise, <c>false</c>.</value>
        public bool WssSiteOnline
        {
            get
            {               
                return _wssSiteOnline;

            }
        }


        public List<DocumentList> DocumentList
        {
            get
            {
                Load();
                return _docList;
            }
        }


        #region IEquatable<Project> Members

        public bool Equals(Project other)
        {
            return (_server.Uri == other._server.Uri) && (_name == other._name);
        }

        #endregion

        #region IWorkItemCoreValue Members

        public object GetWorkItemCoreValue()
        {
            return _project.Name;
        }

        #endregion


    


        /// <summary>
        /// Returns an iteration node from its id
        /// </summary>
        /// <param name="nodeId"></param>
        /// <returns></returns>
        public Node GetIterationNodeFromId(int nodeId)
        {
            Load();
            return Node.Find(_iterationNodeList, nodeId);
        }

        /// <summary>
        /// Returns an area node from its id
        /// </summary>
        /// <param name="nodeId"></param>
        /// <returns></returns>
        public Node GetAreaNodeFromId(int nodeId)
        {
            Load();
            return Node.Find(_areaNodeList, nodeId);
        }
    }
}
