using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Text;
using System.Linq;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Server;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSServer = Microsoft.TeamFoundation.Server;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;
using TFSVersionControl = Microsoft.TeamFoundation.VersionControl.Client;


namespace Fissum.TeamSystem
{
    public delegate void TFSConnectDelegate();


    

    public sealed class TeamFoundationServer: IDisposable, IEquatable<TeamFoundationServer>
    {
        #region consts
        private const string WSSSiteBaseUrlKey = "BaseSiteUrl";
        private const string WSSSiteBaseUNCKey = "BaseSiteUnc";
  

        #endregion
        #region Statics

        public delegate NetworkCredential GetCustomCredentialDelegate(Uri serverUrl);

        /// <summary>
        /// Gets the servers registrated in the team explorer.
        /// </summary>
        /// <returns></returns>
        public static List<Uri> GetAvailableServers()
        {
            return (from TFSClient.TeamFoundationServer srv in TFSClient.RegisteredServers.GetServers()
                    select srv.Uri).ToList();
        }

        /// <summary>
        /// Gets the servers.
        /// </summary>
        /// <returns></returns>
        /// <param name="allowUI"></param>
        public static List<TeamFoundationServer> GetRegisteredServers(GetCustomCredentialDelegate customCredentialDelegate,Func<string,bool> predicate, bool allowUI)
        {
            List<TeamFoundationServer> result = new List<TeamFoundationServer>();

            TFSClient.TeamFoundationServer[] servers;
            try
            {
                servers = TFSClient.RegisteredServers.GetServers();
            }
            catch
            {
                return result;
            }
           
            foreach (TFSClient.TeamFoundationServer srv in servers)
            {
                if (predicate != null)
                {
                    if (!predicate(srv.Uri.ToString()))
                    {
                        continue;
                    }
                }
              
                TeamFoundationServer newServer = TeamFoundationServer.Create(srv.Uri, customCredentialDelegate, allowUI);

                result.Add(newServer);
            }
            return result;
        }

        public static TeamFoundationServer Create(Uri url, GetCustomCredentialDelegate customCredentialDelegate, bool allowUI)
        {
            NetworkCredential customCredential = null;
            if (customCredentialDelegate != null)
            {
                customCredential = customCredentialDelegate(url);
            }

            TFSClient.TeamFoundationServer tfServer;
            ICredentialsProvider provider = null;

            if (allowUI)
            {
                provider = new TFSClient.UICredentialsProvider();
            }

            if (customCredential == null)
            {
                tfServer = new TFSClient.TeamFoundationServer(url.ToString(), provider);
            }
            else
            {
                tfServer = new TFSClient.TeamFoundationServer(url.ToString(), customCredential, provider);
            }

            return new TeamFoundationServer(tfServer);
        }
        #endregion       

        private TFSClient.TeamFoundationServer _tfsServer;
        private TFSTracking.WorkItemStore _globalStore;   
        private string _userName;
        private string _wssSiteBaseUrl;
        private string _wssSiteBaseUnc;

        private List<Project> _projectList;
        private Object _lock;

        public delegate void TeamFoundationServerEventHandler(TeamFoundationServer server);

        public static event TeamFoundationServerEventHandler ConnectionOpened;
        public static event TeamFoundationServerEventHandler ConnectionClosed;
        public static event TeamFoundationServerEventHandler ConnectionFailed;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:TeamFoundationServer"/> class.
        /// </summary>
        /// <param name="server">The server.</param>
        internal  TeamFoundationServer(TFSClient.TeamFoundationServer server)
        {
            _tfsServer = server;
            _projectList = new List<Project>();
            _lock = new object();            
        }       

        /// <summary>
        /// Connects this instance.
        /// </summary>
        /// <returns></returns>
        public bool Connect()
        {            
            try
            {
                _tfsServer.Authenticate();

                _globalStore = new TFSTracking.WorkItemStore(_tfsServer);
               _userName = _tfsServer.AuthenticatedUserName;
               
#if (false)
              
                TFSServer.ICommonStructureService commonStructureService =
                   (TFSServer.ICommonStructureService)_tfsServer.GetService(typeof(TFSServer.ICommonStructureService));
              
                ProjectInfo [] myprojects = commonStructureService.ListProjects();
                
                foreach (ProjectInfo pinfo in myprojects)
                {
                    TFSTracking.Project proj = _globalStore.Projects[pinfo.Name];
                    if (proj != null)
                    {
                        Project project = new Project(this, proj);
                        _projectList.Add(project);  
                    }
                }
#else
               var versionControlServer = (TFSVersionControl.VersionControlServer)_tfsServer.GetService(typeof(TFSVersionControl.VersionControlServer));
               var projects = versionControlServer.GetAllTeamProjects(true);

               foreach (TFSVersionControl.TeamProject pinfo in projects)
               {
                   TFSTracking.Project proj = _globalStore.Projects[pinfo.Name];
                   if (proj != null)
                   {
                       Project project = new Project(this, proj);
                       _projectList.Add(project);
                   }
               }
#endif
              
                ExtractWSSInfo();
                if (ConnectionOpened != null)
                {
                    ConnectionOpened(this);
                }
                return true;
            }
            catch
            {
                
                if (_tfsServer.HasAuthenticated)
                {
                    _tfsServer.Dispose();
                    _tfsServer = null;
                }
                if (ConnectionFailed != null)
                {
                    ConnectionFailed(this);
                }                
            }
            return false;
        }

        private void ExtractWSSInfo()
        {
            TFSServer.IRegistration registrationService = (TFSServer.IRegistration)_tfsServer.GetService(typeof(TFSServer.IRegistration));
            TFSServer.RegistrationEntry[] registrationEntries = registrationService.GetRegistrationEntries("Wss");

            if ((registrationEntries != null) && (registrationEntries.Length > 0))
            {
                foreach (TFSServer.ServiceInterface srvItf in registrationEntries[0].ServiceInterfaces)
                {
                    if (srvItf.Name == WSSSiteBaseUrlKey)
                    {
                        _wssSiteBaseUrl = srvItf.Url;
                    }
                    if (srvItf.Name == WSSSiteBaseUNCKey)
                    {
                        _wssSiteBaseUnc = srvItf.Url;

                    }
                }
            }
        }    
  



        #region Fields

        /// <summary>
        /// Gets the URI.
        /// </summary>
        /// <value>The URI.</value>
        [DebuggerBrowsable(DebuggerBrowsableState.Never)]
        public Uri Uri
        {
            [DebuggerStepThrough]
            get
            {
                if (_tfsServer == null)
                {
                    return null;
                }
                return _tfsServer.Uri;
            }
        }


        internal TFSClient.TeamFoundationServer TFServer
        {
            get
            {
                return _tfsServer;
            }
        }
        /// <summary>
        /// Gets a value indicating whether this <see cref="T:TeamFoundationServer"/> is connected.
        /// </summary>
        /// <value><c>true</c> if connected; otherwise, <c>false</c>.</value>
        [DebuggerBrowsable(DebuggerBrowsableState.Never)]
        public bool Connected
        {
            [DebuggerStepThrough]
            get
            {
                return _tfsServer.HasAuthenticated;
            }
        }

        /// <summary>
        /// Gets the lock.
        /// </summary>
        /// <value>The lock.</value>
        [DebuggerBrowsable(DebuggerBrowsableState.Never)]
        public Object Lock
        {
            [DebuggerStepThrough]
            get
            {
                return _lock;
            }
        }


        /// <summary>
        /// Gets the projects.
        /// </summary>
        /// <value>The projects.</value>
        public Project[] Projects
        {
            get
            {
                return _projectList.ToArray();
            }
        }
        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            _tfsServer.Dispose();
        }


        internal string WssSiteBaseUrl
        {
            get
            {
                return _wssSiteBaseUrl;
            }

        }

        internal string WssSiteBaseUnc
        {
            get
            {
                return _wssSiteBaseUnc;
            }
        }

        #endregion


        /// <summary>
        /// Gets the project.
        /// </summary>
        /// <param name="projectName">Name of the project.</param>
        /// <returns></returns>
        public Project GetProject(string projectName)
        {
            foreach (Project project in _projectList)
            {
                if (project.Name  == projectName)
                {
                    return project;
                }                
            }
            return null;
        }

  

        public WorkItem FindWorkItem(int id)
        {
            try
            {
                TFSTracking.WorkItem wi = _globalStore.GetWorkItem(id);
                if (wi == null)
                {
                    return null;
                }

                Project wiProject = new Project(this, wi.Project);
                return WorkItem.Create(wi, wiProject, false);
            }
            catch
            {
                return null;
            }
        }

        #region IEquatable<TeamFoundationServer> Members

        public bool Equals(TeamFoundationServer other)
        {
            return TFComparer.CompareTFSUri(_tfsServer.Uri, other._tfsServer.Uri) == 0;
        }

        #endregion
    }

}
