﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using Fissum.Core;
using Fissum.Core.Configuration;
using Fissum.TeamSystem;

namespace Fissum.Core
{
    /// <summary>
    /// Global class used by fissum to handle fissum configuration
    /// </summary>
    public class Context
    {

        #region Fields
        private CoreConfiguration _configuration;        
        List<TeamSystem.TeamFoundationServer> _servers;
        #endregion

        #region Constructors
        ///// <summary>
        ///// Constructor
        ///// </summary>
        ///// <param name="mode"></param>
        //public Context(ContextMode mode)
        //{
        //    if (mode == ContextMode.Manual)
        //    {
        //        throw new ArgumentNullException("mode", "Manual mode not supported within this constructor");
        //    }


        //    List<string> assemblyList = new List<string>()
        //    {
        //        "Microsoft.TeamFoundation",
        //        "Microsoft.TeamFoundation.Client",
        //        "Microsoft.TeamFoundation.Common",
        //        "Microsoft.TeamFoundation.WorkItemTracking.Client",             
        //        "Microsoft.TeamFoundation.WorkItemTracking.Client.QueryLanguage",
        //        "Microsoft.TeamFoundation.WorkItemTracking.Client.RuleEngine",
        //        "Microsoft.TeamFoundation.WorkItemTracking.Controls",
        //        "Microsoft.TeamFoundation.VersionControl.Client"
        //    };
        //    Resolver.Init(TFSVersion.TFS2005);
        //    foreach (var assName in assemblyList)
        //    {
        //        Resolver.Instance.LoadTFSAssembly(assName);
        //    }
        //    _mode = mode;
        //}

        public Context(bool allowUI):
            this(CoreConfiguration.LoadFromDiskOrCreateWithFullServerList(ConfigurationFileName,allowUI))
        {

        }

        public Context(Configuration.CoreConfiguration configuration)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException("configuration");
            }
            _configuration = configuration;
            Resolver.Init(TFSVersion.TFS2010);
            _servers = new List<TeamFoundationServer>();
        }
        #endregion

        #region Events
        /// <summary>
        /// Event handler for context events
        /// </summary>
        public event EventHandler<ContextGlobalEvent> ContextEvent;

        /// <summary>
        /// Deletage used to notify the end of the connect sequence
        /// </summary>
        public delegate void EndConnectDelegate();
        #endregion

        #region Methods

        /// <summary>
        /// Checks if the fissum data directory exists, the directory is created if not
        /// </summary>
        public void CheckFissumAPIDirectory()
        {
            DirectoryInfo dirInfo = new DirectoryInfo(DataDirectory);
            if (dirInfo.Exists)
                return;

            dirInfo.Create();
            OnGlobalEvent("Fissum configuration directory created", ContextMessageLevel.Info, ContextMessageType.Configuration);
        }


        /// <summary>
        /// Returns the custom credential according the context configuration
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        private NetworkCredential GetCustomCredential(Uri uri)
        {
            return GetCustomCredentialFromConfiguration(uri);
        }

        /// <summary>
        /// Returns the custom credential stored in the configuration file for a specific server
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        private NetworkCredential GetCustomCredentialFromConfiguration(Uri uri)
        {
            if (_configuration == null)
            {
                return null;
            }

            ServerInfo serverInfo = (from srv in _configuration.ServerList where TFComparer.CompareTFSUri(new Uri(srv.ServerAddress),uri) == 0 select srv).FirstOrDefault();

            if (serverInfo == null || !serverInfo.HasCustomCredential)
            {
                return null;
            }

            string login = serverInfo.GetLogin();
            string password = serverInfo.GetPassword();

            if (login == null)
            {
                return null;
            }
            string[] loginArray = login.Split('\\');

            NetworkCredential result = new NetworkCredential { Password = password };

            if (loginArray.Length == 1)
            {
                result.Domain = String.Empty;
                result.UserName = loginArray[0];
            }
            else if (loginArray.Length == 2)
            {
                result.Domain = loginArray[0];
                result.UserName = loginArray[1];
            }
            else
            {
                return null;
            }

            return result;
        }

        /// <summary>
        /// Gets a project of a from its name and a server URL
        /// </summary>
        /// <param name="tfsName">Name of the TFS.</param>
        /// <param name="projectName">Name of the project.</param>
        /// <returns></returns>
        public Project GetProject(string tfsName, string projectName)
        {
            TeamFoundationServer srv = GetServer(tfsName);
            if (srv == null)
            {
                return null;
            }

            if (!srv.Connected)
            {
                return null;
            }

            return srv.GetProject(projectName);
        }
        /// <summary>
        /// throws a ContextEvent
        /// </summary>
        /// <param name="message"></param>
        private void OnGlobalEvent(string message, ContextMessageLevel level, ContextMessageType type)
        {
            if (String.IsNullOrEmpty(message))
            {
                throw new ArgumentNullException("message");
            }

            if (ContextEvent != null)
            {
                ContextEvent(this, new ContextGlobalEvent() { Message = message, Level = level, Type = type });
            }
        }

        /// <summary>
        /// throws a ContextEvent with several parameters
        /// </summary>
        /// <param name="message"></param>
        /// <param name="level"></param>
        /// <param name="type"></param>
        /// <param name="messageParams"></param>
        private void OnGlobalEvent(string message, ContextMessageLevel level, ContextMessageType type, params Object[] messageParams)
        {
            OnGlobalEvent(String.Format(message, messageParams), level, type);
        }

        public bool SaveConfiguration()
        {
            bool saved;
            try
            {
                CheckFissumAPIDirectory();
                saved = _configuration.SaveToDisk(ConfigurationFileName);
            }
            catch
            {
                saved = false;
            }

            if (saved)
            {
                OnGlobalEvent(Resources.CONF_Saved, ContextMessageLevel.Info, ContextMessageType.Configuration);
            }
            else
            {
                OnGlobalEvent(Resources.CONF_SaveError, ContextMessageLevel.Error, ContextMessageType.Configuration);
            }
            return saved;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Returns the path to the fissum configuration path
        /// </summary>
        public static string DataDirectory
        {
            get
            {
                string appDataPath = System.Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                return Path.Combine(appDataPath, "FissumAPI.2005");
            }
        }


        /// <summary>
        /// Returns true whether the configuration is loaded or not
        /// </summary>
        public bool IsConfigurationLoaded
        {
            get
            {
                return _configuration != null;
            }
        }
        #endregion

        #region Static Properties
        /// <summary>
        /// return the name of the fissum configuration file
        /// </summary>
        internal static string ConfigurationFileName
        {
            get
            {
                return Path.Combine(DataDirectory, "core.config");
            }
        }
        #endregion
        public bool IsConnectedAtStartup(TeamFoundationServer server)
        {
            return _configuration.ServerList.Any(
                    srv => srv.AutoConnect && 
                           TFComparer.CompareTFSUrl(srv.ServerAddress,server.Uri.ToString()) == 0);
        }

        /// <summary>
        /// Launches the connect sequence for servers marked as autoconnect async
        /// </summary>
        /// <param name="allowUI"></param>
        public void Connect()
        {         
            using (AutoResetEvent waitHandle = new AutoResetEvent(false))
            {
                this.ConnectAsync(() => waitHandle.Set());
                waitHandle.WaitOne();
            }
         }

        /// <summary>
        /// Launches the connect sequence for servers marked as autoconnect async
        /// </summary>
        /// <param name="allowUI"></param>
        /// <param name="endDelegate"></param>
        public void ConnectAsync(Action endDelegate)
        {
            if (endDelegate == null)
            {
                throw new ArgumentNullException("endDelegate");
            }

            foreach (var serverInfo in _configuration.ServerList)
            {
                TeamFoundationServer server = TeamFoundationServer.Create(new Uri(serverInfo.ServerAddress), GetCustomCredential, _configuration.AllowUI);

                _servers.Add(server);
            }

            if (_configuration.MergeWithTeamExplorerServers)
            {
                foreach (var srv in TeamFoundationServer.GetAvailableServers())
                {
                    if (_servers.Any(registeredServer => TFComparer.CompareTFSUri(srv, registeredServer.Uri) == 0))
                    {
                        continue;
                    }
                    TeamFoundationServer server = TeamFoundationServer.Create(srv, GetCustomCredential, _configuration.AllowUI);

                    _servers.Add(server);
                }
            }

            ThreadPool.QueueUserWorkItem(
                delegate
                {
                    _servers.ForEach(server =>
                    {
                        if (IsConnectedAtStartup(server))
                        {
                            lock (server)
                            {
                                OnGlobalEvent(Resources.TFS_ConnectionAttempt, ContextMessageLevel.Info, ContextMessageType.TFS, server.Uri);
                                server.Connect();
                            }
                        }
                    });
                    endDelegate();
                });
        }

        /// <summary>
        /// Gets a server from a given url
        /// </summary>
        /// <param name="tfsName">Name of the TFS.</param>
        /// <returns></returns>
        public TeamFoundationServer GetServer(string tfsName)
        {
            foreach (TeamFoundationServer server in _servers)
            {
                if (server.Uri == null)
                {
                    continue;
                }
                if (server.Uri.ToString() == tfsName)
                {
                    return server;
                }
            }
            return null;
        }

        /// <summary>
        /// Gets the registered servers.
        /// </summary>
        /// <value>The servers.</value>
        public IEnumerable<TeamFoundationServer> Servers
        {
            get
            {
                return _servers.ToArray();
            }
        }

        /// <summary>
        /// Gets or sets the current configuration
        /// </summary>
        /// <value>The configuration.</value>
        internal CoreConfiguration Configuration
        {
            get
            {
                return _configuration;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                _configuration = value;
            }
        }

        public bool IsTeamExplorerInstalled
        {
            get
            {
                return Resolver.Instance.IsTeamExplorerInstalled;
            }
        }
    }
}
