﻿using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using Fissum.Core.Linq;
using Fissum.TeamSystem;
using Fissum.TeamSystem.Model;
using Microsoft.CSharp;

namespace Fissum.Model.Generator
{
    public enum FieldSeparator { NoSpace, Underscore }

    /// <summary>
    /// Class used to generate the model
    /// </summary>
    public class Engine
    {
        private List<string> _fieldToIgnoreList;
        private List<string> _definedPropertyList;
        private FieldSeparator _fieldSeparator = FieldSeparator.NoSpace;

        public FieldSeparator FieldSeparator
        {
            get
            {
                return _fieldSeparator;
            }
            set
            {
                if (_fieldSeparator == value)
                {
                    return;
                }
                _fieldSeparator = value;
            }
        }

        public Engine()
        {
            _fieldToIgnoreList = new List<string>();
            _definedPropertyList = new List<string>();

            foreach (var pi in typeof(WorkItem).GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                _definedPropertyList.Add(pi.Name);
                var attribs = pi.GetCustomAttributes(typeof(FieldAttribute), false) as FieldAttribute[];

                if (attribs == null || attribs.Length != 1)
                {
                    continue;
                }

                _fieldToIgnoreList.Add(attribs[0].ReferenceName);
            }
        }

        /// <summary>
        /// Method used to generate model from a modelDefinition
        /// </summary>
        /// <param name="modeldefinition"></param>
        /// <param name="codeProvider"></param>
        /// <returns></returns>
        public bool GenerateModel(ModelDefinition modeldefinition,CodeDomProvider codeProvider,TextWriter output)
        {
            CodeCompileUnit unit = new CodeCompileUnit();

            // Declare a new namespace called Samples.
            CodeNamespace codeNamespace = new CodeNamespace(modeldefinition.Namespace);
            // Add the new namespace to the compile unit.
            unit.Namespaces.Add(codeNamespace);


            foreach (var classDefinition in modeldefinition.ClassDefinitions)
            {
                CodeTypeDeclaration wiTypeClass = GenerateWorkItemTypeClass(classDefinition);
                codeNamespace.Types.Add(wiTypeClass);

                CodeTypeDeclaration wiCreatorProviderType = GenerateWorkItemCreatorProviderClass(classDefinition);
                codeNamespace.Types.Add(wiCreatorProviderType);

            }

            CodeGeneratorOptions options = new CodeGeneratorOptions();
            options.BlankLinesBetweenMembers = true;
            options.IndentString = "  ";
             
            try
            {
                codeProvider.GenerateCodeFromCompileUnit(unit, output, options);
                output.Flush();
            }
            catch
            {
                return false;
            }

            return true;
        }

        private CodeTypeDeclaration GenerateWorkItemTypeClass(ModelClassDefinition classDefinition)
        {
            CodeTypeDeclaration wiTypeClass = new CodeTypeDeclaration(classDefinition.ClassName);
            wiTypeClass.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            wiTypeClass.IsPartial = true;
            wiTypeClass.IsClass = true;
            wiTypeClass.BaseTypes.Add(typeof(WorkItem));

            wiTypeClass.CustomAttributes.Add(new CodeAttributeDeclaration(
                new CodeTypeReference(typeof(ModelTypeAttribute)),
                new CodeAttributeArgument(new CodePrimitiveExpression(classDefinition.WorkItemType))));

            wiTypeClass.CustomAttributes.Add(new CodeAttributeDeclaration(
                new CodeTypeReference(typeof(CreatorProviderAttribute)),
                new CodeAttributeArgument(new CodeTypeOfExpression(classDefinition.ClassName + "Provider"))));

            if (!String.IsNullOrEmpty(classDefinition.Description))
            {

                CodeComment comment = new CodeComment(String.Format("<summary>{0}</summary>", classDefinition.Description), true);
                wiTypeClass.Comments.Add(new CodeCommentStatement(comment));
            }


            GenerateConstructor(wiTypeClass);

            var sortedFields = from field in classDefinition.FieldDefinitions
                               orderby field.Name
                               select field;

            foreach (var fieldDefinition in sortedFields)
            {
                if (_fieldToIgnoreList.Contains(fieldDefinition.ReferenceName))
                {
                    continue;
                }

                GenerateField(wiTypeClass, fieldDefinition);
            }
            return wiTypeClass;
        }

        /// <summary>
        /// Generates the work item creator provider class.
        /// </summary>
        /// <param name="classDefinition">The class definition.</param>
        /// <returns></returns>
        private CodeTypeDeclaration GenerateWorkItemCreatorProviderClass(ModelClassDefinition classDefinition)
        {
            CodeTypeDeclaration wiTypeProviderClass = new CodeTypeDeclaration(classDefinition.ClassName + "Provider");
            wiTypeProviderClass.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            wiTypeProviderClass.IsPartial = false;
            wiTypeProviderClass.IsClass = true;

            CodeTypeReference interfaceReference = new CodeTypeReference("Fissum.Core.Linq.IWorkItemCreatorProvider");
            interfaceReference.TypeArguments.Add(classDefinition.ClassName);
            wiTypeProviderClass.BaseTypes.Add(interfaceReference);

            GenerateCreateMethod(wiTypeProviderClass, classDefinition);

            return wiTypeProviderClass;
        }

        /// <summary>
        /// Generates the create method of the provider class
        /// </summary>
        /// <param name="WiTypeProviderClass">The wi type provider class.</param>
        /// <param name="classDefinition">The class definition.</param>
        private void GenerateCreateMethod(CodeTypeDeclaration WiTypeProviderClass, ModelClassDefinition classDefinition)
        {
            CodeMemberMethod method = new CodeMemberMethod();
            method.Name = "CreateItem";
            method.Attributes = MemberAttributes.Public;
            method.Parameters.Add(new CodeParameterDeclarationExpression("Microsoft.TeamFoundation.WorkItemTracking.Client.WorkItem", "workitem"));
            method.Parameters.Add(new CodeParameterDeclarationExpression(typeof(TeamFoundationServer), "server"));
            method.ReturnType = new CodeTypeReference(classDefinition.ClassName);

            CodeObjectCreateExpression constructorInvoke = new CodeObjectCreateExpression(
                new CodeTypeReference(classDefinition.ClassName), 
                new CodeExpression[] 
                { 
                    new CodeVariableReferenceExpression("workitem"), 
                    new CodeVariableReferenceExpression("server") 
                });

            method.Statements.Add(new CodeMethodReturnStatement(constructorInvoke));

            WiTypeProviderClass.Members.Add(method);
        }
        /// <summary>
        /// Generates a field.
        /// </summary>
        /// <param name="wiTypeClass">The wi type class.</param>
        /// <param name="fieldDefinition">The field definition.</param>
        private void GenerateField(CodeTypeDeclaration wiTypeClass, ModelFieldDefinition fieldDefinition)
        {
            CodeMemberProperty property = new CodeMemberProperty();
            property.HasGet = true;

            string PropName = GeneratePropertyName(fieldDefinition);

            property.Name = PropName;

            Type fieldType = Type.GetType(fieldDefinition.TypeName);
            if (!fieldType.IsClass)
            {
                Type nullableType = typeof(Nullable<>);
                fieldType = nullableType.MakeGenericType(fieldType);

            }
            property.Type = new CodeTypeReference(fieldType);
            property.Attributes = MemberAttributes.Public;
        
            property.CustomAttributes.Add(new CodeAttributeDeclaration(
                   new CodeTypeReference(typeof(FieldAttribute)),
                   new CodeAttributeArgument(new CodePrimitiveExpression(fieldDefinition.ReferenceName))));


            if (!String.IsNullOrEmpty(fieldDefinition.Description))
            {

                CodeComment comment = new CodeComment(String.Format("<summary>{0}</summary>",fieldDefinition.Description), true);
                property.Comments.Add(new CodeCommentStatement(comment));
            }

            CodeMethodReferenceExpression fieldMethod = new CodeMethodReferenceExpression(new CodeThisReferenceExpression(), "Field", new CodeTypeReference[] { new CodeTypeReference(fieldType) });
            CodeMethodInvokeExpression fieldMethodInvoke = new CodeMethodInvokeExpression(fieldMethod, new CodeExpression[] { new CodePrimitiveExpression(fieldDefinition.ReferenceName) });

            property.GetStatements.Add(new CodeMethodReturnStatement(fieldMethodInvoke));


            if (!fieldDefinition.IsReadOnly)
            {

                CodeMethodReferenceExpression setFieldMethod = new CodeMethodReferenceExpression(new CodeThisReferenceExpression(), "SetField", new CodeTypeReference[] { new CodeTypeReference(fieldType) });
                CodeMethodInvokeExpression setFielMethodInvoke = new CodeMethodInvokeExpression(setFieldMethod, new CodeExpression[] { new CodePrimitiveExpression(fieldDefinition.ReferenceName), new CodePropertySetValueReferenceExpression() });
                property.SetStatements.Add(setFielMethodInvoke);
            }
            wiTypeClass.Members.Add(property);
            _definedPropertyList.Add(property.Name);
        }

        /// <summary>
        /// Generates the name of the property.
        /// </summary>
        /// <param name="fieldDefinition">The field definition.</param>
        /// <returns></returns>
        private string GeneratePropertyName(ModelFieldDefinition fieldDefinition)
        {
            if (!String.IsNullOrEmpty(fieldDefinition.PropertyName))
            {
                return fieldDefinition.PropertyName;
            }

            string replacement;
            
            switch (_fieldSeparator)
            {
                case FieldSeparator.NoSpace:
                    replacement = String.Empty;
                    break;
                case FieldSeparator.Underscore:
                    replacement = "_";
                    break;
                default:
                    throw new GenerationException(ErrorMessageRessource.InvalidSeparatorToken);
            }

            string propName = Regex.Replace(fieldDefinition.Name, @"\W+", replacement);

            if (_definedPropertyList.Contains(propName))
            {
                throw new GenerationException(ErrorMessageRessource.PropertyNameConflict, fieldDefinition.ReferenceName);
            }

            return propName;
        }

        private static void GenerateConstructor(CodeTypeDeclaration wiTypeClass)
        {
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.FamilyAndAssembly;
            constructor.Parameters.Add(new CodeParameterDeclarationExpression("Microsoft.TeamFoundation.WorkItemTracking.Client.WorkItem", "workitem"));
            constructor.Parameters.Add(new CodeParameterDeclarationExpression(typeof(TeamFoundationServer), "server"));

            constructor.BaseConstructorArgs.Add(new CodeVariableReferenceExpression("workitem"));
            constructor.BaseConstructorArgs.Add(new CodeVariableReferenceExpression("server"));
            wiTypeClass.Members.Add(constructor);
        }
    }
}
