using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.TeamSystem.Collections
{
    /// <summary>
    /// Represent a collection of workitem 
    /// </summary>
    public class WorkItemCollection : IList<WorkItem>
    {
        private List<WorkItem> _workItemList;
        private Dictionary<WorkItem, WorkItemCollectionStatus> _statusDictionnary; 


//        protected List<WorkItemObject> WorkItemList;

        public WorkItemCollection()
        {
            _workItemList = new List<WorkItem>();
            _statusDictionnary = new Dictionary<WorkItem, WorkItemCollectionStatus>();
        }

        public WorkItemCollection(IEnumerable<WorkItem> source)
        {
            _workItemList = new List<WorkItem>(source);
        }

        public WorkItemCollection Clone()
        {
            WorkItemCollection result = new WorkItemCollection();
            foreach (WorkItem wi in _workItemList)
            {
                result.Add(wi);
                WorkItemCollectionStatus status;

                if (_statusDictionnary.TryGetValue(wi, out status))
                {
                    result.SetStatus(wi,status);
                }
            }
            return result;
        }


        #region IList<WorkItemObject> Members

        public int IndexOf(WorkItem item)
        {
            return _workItemList.IndexOf(item);
        }

        public void Insert(int index, WorkItem item)
        {
            _workItemList.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            WorkItem wi = this[index];
            _statusDictionnary.Remove(wi);
            _workItemList.RemoveAt(index);
        }

        public WorkItem this[int index]
        {
            get
            {
                return _workItemList[index];
            }
            set
            {
                _workItemList[index] = value;
            }
        }

        #endregion

        #region ICollection<WorkItemObject> Members

        public void Add(WorkItem item)
        {
            _workItemList.Add(item);
        }

        public void Add(WorkItem item, WorkItemCollectionStatus status)
        {
            _workItemList.Add(item);
            _statusDictionnary.Add(item, status);
        }

        public void Clear()
        {
            _workItemList.Clear();
            _statusDictionnary.Clear();
        }

        public bool Contains(WorkItem item)
        {
            return _workItemList.Contains(item);
        }

        public void CopyTo(WorkItem[] array, int arrayIndex)
        {
            _workItemList.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return _workItemList.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(WorkItem item)
        {
            _statusDictionnary.Remove(item);
           return  _workItemList.Remove(item);
        }

     
        #endregion

        #region IEnumerable<WorkItemObject> Members

        public IEnumerator<WorkItem> GetEnumerator()
        {
            return _workItemList.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion


        #region Status management
        /// <summary>
        /// Sets the status of a work item in the list
        /// </summary>
        /// <param name="wi"></param>
        /// <param name="status"></param>
        public void SetStatus(WorkItem wi, WorkItemCollectionStatus status)
        {
            if (_workItemList.Contains(wi))
            {
                throw new ArgumentNullException("wi");
            }
            _statusDictionnary[wi] = status;
        }

        /// <summary>
        /// Returns the status of a work item in the list
        /// </summary>
        /// <param name="wi"></param>
        /// <returns></returns>
        public WorkItemCollectionStatus GetStatus(WorkItem wi)
        {
            WorkItemCollectionStatus result = WorkItemCollectionStatus.None;

            _statusDictionnary.TryGetValue(wi, out result);
            return result;
        }

        /// <summary>
        /// Clear the status of all the work items
        /// </summary>
        public void ClearStatus()
        {
            _statusDictionnary.Clear();
        }

        /// <summary>
        /// Returns a IEnumerable collection of workItem according its type
        /// </summary>
        /// <param name="status"></param>
        /// <returns></returns>
        public List<WorkItem> GetWorkItemFromStatus(WorkItemCollectionStatus status)
        {
            return (from wi in _workItemList
                    where GetStatus(wi) == status
                    select wi).ToList();
        }

        #endregion

        #region Comparaison

        /// <summary>
        /// Compare two list of work items and change the status of the work item is the list
        /// </summary>
        /// <param name="leftCollection"></param>
        /// <param name="rightCollection"></param>
        /// <returns></returns>
        public static WorkItemCollection Diff(WorkItemCollection leftCollection, WorkItemCollection rightCollection)
        {
            WorkItemCollection result = new WorkItemCollection();

            Dictionary<WorkItem, WorkItemCollectionStatus> workingDictionnary;

            workingDictionnary = new Dictionary<WorkItem, WorkItemCollectionStatus>();

            foreach (var wi in leftCollection)
            {
                workingDictionnary.Add(wi, WorkItemCollectionStatus.Deleted);
            }

            foreach (var rightWi in rightCollection)
            {
                WorkItem leftWi = null;

                leftWi = workingDictionnary.Keys.First(wi => wi == rightWi);

                if (leftWi == null)
                {
                    workingDictionnary.Add(rightWi, WorkItemCollectionStatus.Added);
                }
                else
                {
                    if (leftWi.Revision < rightWi.Revision)
                    {
                        workingDictionnary.Remove(leftWi);
                        workingDictionnary.Add(rightWi, WorkItemCollectionStatus.Modified);
                    }
                    else if (leftWi.Revision > rightWi.Revision)
                    {
                        workingDictionnary[leftWi] = WorkItemCollectionStatus.Modified;
                    }
                    else
                    {
                        workingDictionnary[leftWi] = WorkItemCollectionStatus.None;
                    }
                }
            }

            foreach (var pair in workingDictionnary)
            {
                result.Add(pair.Key, pair.Value);
            }

            return result;
        }

        #endregion

        public void Reverse()
        {
            _workItemList.Reverse();
        }
    }
}
