using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Threading;
using Microsoft.Win32;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Fissum.TeamSystem
{
    /// <summary>
    /// This class is used to resolve TFS assemblies
    /// </summary>
    public class Resolver
    {
        private TFSVersion _version;
        private static Resolver _instance = null;
        public static Resolver Instance
        {
            get
            {
                return _instance;
            }
        }

        private const string REGVS2010KEY = @"SOFTWARE\Microsoft\VisualStudio\10.0";
        private const string REGVS2008KEY = @"SOFTWARE\Microsoft\VisualStudio\9.0";
        private const string REGVS2005KEY = @"SOFTWARE\Microsoft\VisualStudio\8.0";
        private const string REGTE2010KEY = @"SOFTWARE\Microsoft\VisualStudio\10.0\InstalledProducts\Team Explorer";
        private const string REGTE2008KEY = @"SOFTWARE\Microsoft\VisualStudio\9.0\InstalledProducts\Team Explorer";
        private const string REGTE2005KEY = @"SOFTWARE\Microsoft\VisualStudio\8.0\InstalledProducts\Team Explorer";

        private const string TFSASSEMBLYSTARTNAME = "Microsoft.TeamFoundation";

        public static void Init(TFSVersion version)
        {
            if (_instance != null)
            {
                return;
            }

            _instance = new Resolver(version);

            
        }

        private Assembly AssemblyResolveEvent(object sender, ResolveEventArgs args)
        {
            AssemblyName name = new AssemblyName(args.Name);
            return LoadTFSAssembly(name.Name);
        }

        public Assembly LoadTFSAssembly(string assemblyName)
        {         
            if (assemblyName.StartsWith(TFSASSEMBLYSTARTNAME))
            {
            

                string tfsPath = GetTFSAssemblyPath();


                string fullPath = Path.Combine(tfsPath, assemblyName) + ".dll";
                if (File.Exists(fullPath))
                {                  
                    Assembly loadedAssembly = Assembly.LoadFile(fullPath);
                    return loadedAssembly;
                }
            }
            return null;
        }

        public string GetTFSAssemblyPath()
        {
            if (_version == TFSVersion.TFS2010)
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(REGVS2010KEY);

                if (key != null)
                {
                    using (key)
                    {
                        string vspath = key.GetValue("InstallDir", null) as string;

                        if (vspath == null)
                        {
                            throw new InvalidOperationException("VS2010: invalid configuration");
                        }

                        return Path.Combine(vspath, "PrivateAssemblies");
                    }
                }
            }
            else if (_version == TFSVersion.TFS2008)
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(REGVS2008KEY);
                
                if (key != null)
                {
                    using (key)
                    {
                        string vspath = key.GetValue("InstallDir", null) as string;

                        if (vspath == null)
                        {
                            throw new InvalidOperationException("VS2008: invalid configuration");
                        }
                       
                        return Path.Combine(vspath, "PrivateAssemblies");
                    }
                }
            }
            else if (_version == TFSVersion.TFS2005)
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(REGVS2005KEY);

                if (key != null)
                {
                    using (key)
                    {
                        string vspath = key.GetValue("InstallDir", null) as string;

                        if (vspath == null)
                        {
                            throw new InvalidOperationException("VS2005: invalid configuration");
                        }                      
                        return Path.Combine(vspath, "PrivateAssemblies");
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// Gets a value indicating whether this instance is team explorer installed.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is team explorer installed; otherwise, <c>false</c>.
        /// </value>
        public bool IsTeamExplorerInstalled
        {
            get
            {
                string name;
                switch (_version)
                {
                    case TFSVersion.TFS2005:
                        name = REGTE2005KEY;
                        break;
                    case TFSVersion.TFS2008:
                        name = REGTE2008KEY;
                        break;
                    case TFSVersion.TFS2010:
                        name = REGTE2010KEY;
                        break;
                    default:
                        return false;
                        break;
                }
                var key = Registry.LocalMachine.OpenSubKey(name);
                if (key == null)
                {
                    return false;
                }
                key.Close();
                return true;
            }
        }
        public Resolver(TFSVersion version)
        {
            _version = version;
            AppDomain currentDomain = AppDomain.CurrentDomain;
            currentDomain.AssemblyResolve += new ResolveEventHandler(AssemblyResolveEvent);
        }
    }
}
