using System;
using System.Collections.Generic;
using System.Text;
using TFSClient = Microsoft.TeamFoundation.Client;
using TFSTracking = Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Net;
using System.Diagnostics;
using System.Collections;
using System.Runtime.Serialization;
using Fissum.TeamSystem.Collections;
using System.Linq;

namespace Fissum.TeamSystem
{
    public delegate void WorkItemQueryUpdatedEventHandler(WorkItemQueryUpdatedEventArgs args);

    public sealed class WorkItemQuery : IEquatable<WorkItemQuery>
    {
        private TFSTracking.StoredQuery _query;

        private List<WorkItemFieldDefinition> _fieldList;
        private List<WorkItemSortField> _sortList;
        private WorkItemCollection _queryList;
        private Project _project;
        private object _syncRoot = new object();
        private TeamFoundationServer _server;
        private Dictionary<string,object> _defaultParameters = null;


        public event EventHandler<WorkItemQueryUpdatedEventArgs> WorkItemQueryUpdated;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:WorkItemQuery"/> class.
        /// </summary>
        /// <param name="query">The query.</param>
        internal WorkItemQuery(TFSTracking.StoredQuery query, Project project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            _query = query;
            _project = project;
            _server = _project.Server;
        }

        public WorkItemQuery(TeamFoundationServer server, Project project, string title, string wiql, Dictionary<string, object> defaultParameters)
        {
            if (server == null)
            {
                throw new ArgumentNullException("server");
            }
            _server = server;
            _query = new Microsoft.TeamFoundation.WorkItemTracking.Client.StoredQuery(TFSTracking.QueryScope.Private, title, wiql, String.Empty);
            _project = project;
            _defaultParameters = defaultParameters;
        }


        /// <summary>
        /// Gets the TFS query.
        /// </summary>
        /// <value>The TFS query.</value>
        internal TFSTracking.StoredQuery TFSQuery
        {
            get
            {
                return _query;
            }
        }
        /// <summary>
        /// Gets the WIQL.
        /// </summary>
        /// <value>The WIQL.</value>
        public string WIQL
        {
            get
            {
                return _query.QueryText;
            }
        }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                TFSTracking.StoredQuery storedQuery = _query as TFSTracking.StoredQuery;
                if (storedQuery == null)
                {
                    return String.Empty;
                }

                return storedQuery.Name;
            }
        }

        public WorkItemFieldDefinition[] Fields
        {
            get
            {
                if (_fieldList == null)
                {
                    return null;
                }
                return _fieldList.ToArray();
            }
        }

        public WorkItemSortField[] SortFields
        {
            get
            {
                return _sortList.ToArray();
            }
        }

        public bool IsTeamQuery
        {
            get
            {              
                return _query.QueryScope == Microsoft.TeamFoundation.WorkItemTracking.Client.QueryScope.Public;
            }
        }

        public bool IsSaved
        {
            get
            {
                return _query.IsSaved;
            }
        }

        public Project Project
        {
            get
            {
                return _project;
            }
        }

        public TeamFoundationServer Server
        {
            get
            {
                return _server;
            }
        }

        /// <summary>
        /// Does the query.
        /// </summary>
        /// <returns></returns>
        private void DoQuery(
            out WorkItemCollection queryList,
            out List<WorkItemFieldDefinition> fieldList,
            out List<WorkItemSortField> sortList)
        {
            queryList = new WorkItemCollection();
            fieldList = new List<WorkItemFieldDefinition>();
            sortList = new List<WorkItemSortField>();

            Hashtable parameters = new Hashtable();
            if (_project != null)
            {
                parameters.Add("project", _project.Name);
            }

            if (_defaultParameters != null)
            {
                foreach (var item in _defaultParameters)
                {
                    parameters.Add(item.Key, item.Value);
                }
            }

            TFSTracking.WorkItemStore store = new TFSTracking.WorkItemStore(_server.TFServer);
            TFSTracking.WorkItemCollection coll = store.Query(_query.QueryText, parameters);


            foreach (TFSTracking.WorkItem wi in coll)
            {
                Project proj = _server.GetProject(wi.Project.Name);
                queryList.Add(WorkItemObject.Create(wi, proj, false));
            }

            foreach (TFSTracking.FieldDefinition field in coll.DisplayFields)
            {
                fieldList.Add(new WorkItemFieldDefinition(field));
            }

            foreach (TFSTracking.SortField sField in coll.SortFields)
            {
                if (sField.SortType == TFSTracking.SortType.Ascending)
                {
                    sortList.Add(new WorkItemSortField(sField.FieldDefinition, SoftFieldEnum.Ascending));
                }
                else
                {
                    sortList.Add(new WorkItemSortField(sField.FieldDefinition, SoftFieldEnum.Descending));
                }
            }
        }

        /// <summary>
        /// Gets the work items.
        /// </summary>
        /// <returns></returns>
        public WorkItemObject[] GetWorkItems()
        {
           
            if (_queryList == null)
            {
                SynchronizeCore(false);
            }

            return _queryList.ToArray();            
        }

        /// <summary>
        /// Synchronizes this instance.
        /// </summary>
        /// <returns></returns>
        private void SynchronizeCore(bool launchEvent)
        {
            lock (_syncRoot)
            {
                DoQuery(out _queryList, out _fieldList, out _sortList);
                if (launchEvent && (WorkItemQueryUpdated != null))
                {
                    WorkItemQueryUpdated(this, new WorkItemQueryUpdatedEventArgs(this));
                }
            }

        }

        public void Synchronize()
        {
            SynchronizeCore(true);
        }

        #region IEquatable<WorkItemQuery> Members

        public bool Equals(WorkItemQuery other)
        {
            if (other == null)
            {
                return false;
            }
            return _project.Equals(other._project) && (_query.Name == other._query.Name);
        }

        #endregion
    }
}